@@ -56,7 +56,7 @@
  *  @author Martin Pala <martinp@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: alert.c,v 1.41 2005/02/13 00:00:13 martinp Exp $
+ *  @version \$Id: alert.c,v 1.43 2005/08/14 08:44:48 martinp Exp $
  *  @file
  */
 
@@ -83,7 +83,7 @@ void handle_alert(Event_T E) {
   ASSERT(E);
 
   s= Event_get_source(E);
-  if(s->maillist||Run.maillist) {
+  if(s->maillist || Run.maillist) {
     Mail_T m;
     Mail_T n;
     sigset_t ns, os;
@@ -95,27 +95,42 @@ void handle_alert(Event_T E) {
      */
     for(m= s->maillist; m; m= m->next) {
       
-      if(IS_EVENT_SET(m->events, Event_get_id(E))) {
+      if(
+        /* particular event notification type is allowed for given recipient */
+        IS_EVENT_SET(m->events, Event_get_id(E)) &&
+        (
+          /* 'changed' event type is sent always */
+          E->id == EVENT_CHANGED ||
+          /* state change notification is sent always */
+          E->state_changed       ||
+          /* in the case that the state is failed for more cycles we check
+           * whether we should send the reminder */
+          (
+            E->state && m->reminder && E->count % m->reminder == 0
+          )
+        )
+      )
+      {
 
-	Mail_T tmp= NULL;
+        Mail_T tmp= NULL;
 
-	NEW(tmp);
+        NEW(tmp);
 
-	copy_mail(tmp, m);
+        copy_mail(tmp, m);
 
-	if(Event_get_message(E)) {
-	  tmp->opt_message= xstrdup(Event_get_message(E));
-	}
+        if(Event_get_message(E)) {
+          tmp->opt_message= xstrdup(Event_get_message(E));
+        }
 
-	substitute(&tmp, s->name, EVENT_DESCRIPTION(E),
+        substitute(&tmp, s->name, EVENT_DESCRIPTION(E),
           Event_get_action_description(E));
 
-	replace_bare_linefeed(&tmp);
+        replace_bare_linefeed(&tmp);
 
-	tmp->next= list;
-	list= tmp;
+        tmp->next= list;
+        list= tmp;
 
-	DEBUG("%s notification is sent to %s\n", EVENT_DESCRIPTION(E), m->to);
+        DEBUG("%s notification is sent to %s\n", EVENT_DESCRIPTION(E), m->to);
 
       }
 
@@ -132,31 +147,48 @@ void handle_alert(Event_T E) {
 
       for(n= s->maillist; n; n= n->next) {
         if(IS(m->to, n->to)) {
-	  skip= TRUE;
-	  break;
-	}
+          skip= TRUE;
+          break;
+        }
       }
 
-      if(!skip && IS_EVENT_SET(m->events, Event_get_id(E))) {
+      if(
+        /* the local service alert definition has not overrided the global one */
+        !skip &&
+        /* particular event notification type is allowed for given recipient */
+        IS_EVENT_SET(m->events, Event_get_id(E)) &&
+        (
+          /* 'changed' event type is sent always */
+          E->id == EVENT_CHANGED ||
+          /* state change notification is sent always */
+          E->state_changed       ||
+          /* in the case that the state is failed for more cycles we check
+           * whether we should send the reminder */
+          (
+            E->state && m->reminder && E->count % m->reminder == 0
+          )
+        )
+      )
+      {
 
-	Mail_T tmp= NULL;
+        Mail_T tmp= NULL;
 
-	NEW(tmp);
+        NEW(tmp);
 
-	copy_mail(tmp, m);
+        copy_mail(tmp, m);
 
-	if(Event_get_message(E))
-	  tmp->opt_message= xstrdup(Event_get_message(E));
+        if(Event_get_message(E))
+          tmp->opt_message= xstrdup(Event_get_message(E));
 
-	substitute(&tmp, s->name, EVENT_DESCRIPTION(E),
+        substitute(&tmp, s->name, EVENT_DESCRIPTION(E),
           Event_get_action_description(E));
 
-	replace_bare_linefeed(&tmp);
+        replace_bare_linefeed(&tmp);
 
-	tmp->next= list;
-	list= tmp;
+        tmp->next= list;
+        list= tmp;
 
-	DEBUG("%s notification is sent to %s\n", EVENT_DESCRIPTION(E), m->to);
+        DEBUG("%s notification is sent to %s\n", EVENT_DESCRIPTION(E), m->to);
 
       }
 
