@@ -38,6 +38,7 @@
 
 #include "monitor.h"
 #include "socket.h"
+#include "event.h"
 
 
 /**
@@ -45,7 +46,7 @@
  *
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: collector.c,v 1.6 2005/04/05 19:52:57 martinp Exp $
+ *  @version \$Id: collector.c,v 1.7 2005/08/14 08:44:48 martinp Exp $
  *
  *  @file
  */
@@ -72,7 +73,9 @@ void handle_collector(Event_T E) {
   sigset_t     ns;
   sigset_t     os;
 
-  if(!C)
+  /* The event is send to the collector just once - only in the case
+   * that the state changed */
+  if(!C || (!E->state_changed && E->id != EVENT_CHANGED))
   {
     return;
   }
