@@ -60,7 +60,7 @@
  *  @author Rory Toma, <rory@digeo.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: control.c,v 1.95 2005/04/11 22:44:19 martinp Exp $
+ *  @version \$Id: control.c,v 1.97 2005/09/05 09:51:37 martinp Exp $
  *
  *  @file
  */
@@ -188,6 +188,10 @@ void control_service(const char *S, const char *action) {
         /* Only start if stop succeeded */
         do_start(s);
         do_depend(s, ACTION_START);
+      } else {
+        /* enable monitoring of this service again to allow the restart retry
+         * in the next cycle up to timeout limit */
+        Util_monitorSet(s);
       }
       break;
 
@@ -442,10 +446,10 @@ static void *wait_start(void *service) {
   }
   
   if(!Util_isProcessRunning(s)) {
-    Event_post(s, EVENT_EXEC, TRUE, s->action_EXEC,
+    Event_post(s, EVENT_EXEC, STATE_FAILED, s->action_EXEC,
       "'%s' failed to start", s->name);
   } else {
-    Event_post(s, EVENT_EXEC, FALSE, s->action_EXEC,
+    Event_post(s, EVENT_EXEC, STATE_PASSED, s->action_EXEC,
       "'%s' started", s->name);
   }
 
@@ -480,11 +484,11 @@ static int wait_stop(Service_T s) {
   }
 
   if(Util_isProcessRunning(s)) {
-    Event_post(s, EVENT_EXEC, TRUE, s->action_EXEC,
+    Event_post(s, EVENT_EXEC, STATE_FAILED, s->action_EXEC,
       "'%s' failed to stop", s->name);
     return FALSE;
   } else {
-    Event_post(s, EVENT_EXEC, FALSE, s->action_EXEC,
+    Event_post(s, EVENT_EXEC, STATE_PASSED, s->action_EXEC,
       "'%s' stopped", s->name);
   }
 
