@@ -39,7 +39,7 @@
  *
  * @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  * @author Martin Pala <martinp@tildeslash.com>
- * @version \$Id: event.c,v 1.45 2005/01/06 20:51:49 martinp Exp $
+ * @version \$Id: event.c,v 1.50 2005/09/25 20:59:49 martinp Exp $
  * @file
  */
 
@@ -55,6 +55,7 @@ EventTable_T Event_Table[]= {
   {EVENT_GID,        "GID failed",             "GID passed"},
   {EVENT_ICMP,       "ICMP failed",            "ICMP passed"},
   {EVENT_INVALID,    "Invalid type",           "Type passed"},
+  {EVENT_MATCH,      "Regex match",            "No regex match"},
   {EVENT_NONEXIST,   "Does not exist",         "Exists"},
   {EVENT_PERMISSION, "Permission failed",      "Permission passed"},
   {EVENT_RESOURCE,   "Resource limit matched", "Resource limit passed"},
@@ -81,17 +82,18 @@ static int  handle_action(Event_T, Action_T);
  * Post a new Event
  * @param service The Service the event belongs to
  * @param id The event identification
- * @param state TRUE for failed, FALSE for passed event state
+ * @param state The event state
  * @param action Description of the event action
  * @param s Optional message describing the event
  */
 void Event_post(Service_T service, long id, short state, EventAction_T action,
   char *s, ...) {
 
-  Event_T        e = service->eventlist;
+  Event_T e = service->eventlist;
 
   ASSERT(service);
   ASSERT(action);
+  ASSERT(state == STATE_FAILED || state == STATE_PASSED);
 
   if(e == NULL)
   {
@@ -100,7 +102,7 @@ void Event_post(Service_T service, long id, short state, EventAction_T action,
      * that the error flag is set for the passed event, we will allow it (i.e.
      * event queue was flushed during monit reload and the service was in
      * failed state before reload) */
-    if(!state && !(service->error & id))
+    if(state != STATE_FAILED && !(service->error & id))
       return;
 
     /* Initialize event list and add first event. */
@@ -108,8 +110,8 @@ void Event_post(Service_T service, long id, short state, EventAction_T action,
     e->id = id;
     e->collected = time(NULL);
     e->source = service;
-    e->state = state;
-    e->state_changed = TRUE;
+    e->state = STATE_INIT;
+    e->state_map = state;
     e->action = action;
     if(s)
     {
@@ -135,7 +137,12 @@ void Event_post(Service_T service, long id, short state, EventAction_T action,
         LOCK(e->mutex)
           e->collected = time(NULL);
 
-          /* update the message */
+          /* Shift the existing event flags to the left
+           * and set the first bit based on actual state */
+          e->state_map <<= 1;
+          e->state_map |= state;
+
+          /* Update the message */
           if(s)
           {
             long l;
@@ -147,11 +154,6 @@ void Event_post(Service_T service, long id, short state, EventAction_T action,
             va_end(ap);
           }
 
-          if(e->state != state)
-          {
-            e->state = state;
-            e->state_changed = TRUE;
-          }
         END_LOCK;
 	break;
       }
@@ -163,7 +165,7 @@ void Event_post(Service_T service, long id, short state, EventAction_T action,
     {
       /* Only first failed event can initialize the queue for given event type,
        * thus passed events are ignored until first error */
-      if(!state)
+      if(state != STATE_FAILED)
         return;
 
       /* Event was not found in the pending events list, we will add it. */
@@ -171,8 +173,8 @@ void Event_post(Service_T service, long id, short state, EventAction_T action,
       e->id = id;
       e->collected = time(NULL);
       e->source = service;
-      e->state = state;
-      e->state_changed = TRUE;
+      e->state = STATE_INIT;
+      e->state_map = state;
       e->action = action;
       if(s)
       {
@@ -189,6 +191,19 @@ void Event_post(Service_T service, long id, short state, EventAction_T action,
     }
   }
 
+  e->state_changed = Event_check_state(e, state);
+
+  /* In the case that the state changed, update it and reset the counter */
+  if(e->state_changed)
+  {
+    e->state = state;
+    e->count = 1;
+  }
+  else
+  {
+    e->count++;
+  }
+
   LOCK(e->mutex)
     handle_event(e);
   END_LOCK;
@@ -241,6 +256,53 @@ short Event_get_state(Event_T E) {
 }
 
 
+/**
+ * Return the actual event state based on event state bitmap
+ * and event ratio needed to trigger the state change
+ * @param E An event object
+ * @param S Actual posted state
+ * @return The Event raw state
+ */
+short Event_check_state(Event_T E, short S) {
+
+  int       i;
+  int       count = 0;
+  Action_T  action;
+  long long flag;
+
+  ASSERT(E);
+
+  /* Only the true failed state condition can change the initial state */
+  if(S == STATE_PASSED && E->state == STATE_INIT)
+  {
+    return FALSE;
+  }
+
+  action = (S == STATE_PASSED)?E->action->passed:E->action->failed;
+
+  /* Compare as many bits as cycles able to trigger the action */
+  for(i = 0; i < action->cycles; i++)
+  {
+    /* Check the state of the particular cycle given by the bit position */
+    flag = (E->state_map >> i) & 0x1;
+
+    /* Count occurences of the posted state */
+    if(flag == S)
+    {
+      count++;
+    }
+  }
+
+  if(count >= action->count && S != E->state)
+  {
+    return TRUE;
+  }
+  
+  return FALSE;
+
+}
+
+
 /**
  * Get the Event type
  * @param E An event object
@@ -360,9 +422,10 @@ static void handle_event(Event_T E) {
   ASSERT(E->action->failed);
   ASSERT(E->action->passed);
 
-  /* We will handle only first passed event, recurrent passed events are
+  /* We will handle only first passed event, recurrent passed events
+   * or insufficient passed events during failed service state are
    * ignored. Failed events are handled each time. */
-  if(!E->state_changed && !E->state)
+  if(!E->state_changed && E->state == STATE_PASSED)
   {
     return;
   }
@@ -372,7 +435,7 @@ static void handle_event(Event_T E) {
     log("%s\n", E->message);
   }
 
-  if(E->state)
+  if(E->state == STATE_FAILED)
   {
     E->source->error |= E->id;
     handle_action(E, E->action->failed);
@@ -399,13 +462,9 @@ static int handle_action(Event_T E, Action_T A) {
     return TRUE;
   }
 
-  /* Alert and collector event notification is common action. It is send on
-   * state change only (i.e. once) */
-  if(E->state_changed || (E->id == EVENT_CHANGED))
-  {
-    handle_alert(E);
-    handle_collector(E);
-  }
+  /* Alert and collector event notification are common actions */
+  handle_alert(E);
+  handle_collector(E);
 
   switch(A->id)
   {
