@@ -38,6 +38,7 @@
 #define EVENT_EXEC         0x1000
 #define EVENT_CHANGED      0x2000
 #define EVENT_ICMP         0x4000
+#define EVENT_MATCH        0x8000
 
 #define EVENT_DESCRIPTION(E) Event_get_description(E)
 #define IS_EVENT_SET(value, mask) ((value & mask) != 0)
@@ -63,7 +64,7 @@ extern EventTable_T Event_Table[];
  * 
  * @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  * @author Martin Pala, <martinp@tildeslash.com>
- * @version \$Id: event.h,v 1.20 2005/01/06 20:51:49 martinp Exp $
+ * @version \$Id: event.h,v 1.22 2005/09/05 09:51:37 martinp Exp $
  * @file
  */
 
@@ -71,7 +72,7 @@ extern EventTable_T Event_Table[];
  * Post a new Event
  * @param service The Service the event belongs to
  * @param id The event identification
- * @param state TRUE for failed, FALSE for passed event state
+ * @param state The event state
  * @param action Description of the event action
  * @param s Optional message describing the event
  */
@@ -103,6 +104,16 @@ time_t Event_get_collected(Event_T E);
 short Event_get_state(Event_T E);
 
 
+/**
+ * Return the actual event state based on event state bitmap
+ * and event ratio needed to trigger the state change
+ * @param E An event object
+ * @param S Actual posted state
+ * @return The Event raw state
+ */
+short Event_check_state(Event_T E, short S);
+
+
 /**
  * Get the Event type
  * @param E An event object
