@@ -50,6 +50,7 @@ static void _gc_event(Event_T *);
 static void _gc_inf(Info_T *);
 static void _gcpdl(Dependant_T *);
 static void _gcso(Size_T *);
+static void _gcmatch(Match_T *);
 static void _gcchecksum(Checksum_T *);
 static void _gcperm(Perm_T *);
 static void _gcuid(Uid_T *);
@@ -67,7 +68,7 @@ static void _gc_request(Request_T *);
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: gc.c,v 1.60 2005/01/24 01:04:36 hauk Exp $
+ *  @version \$Id: gc.c,v 1.61 2005/08/05 09:38:09 chopp Exp $
  *
  *  @file
  */
@@ -173,6 +174,9 @@ static void _gc_service(Service_T *s) {
   if((*s)->sizelist)
     _gcso(&(*s)->sizelist);
 
+    if((*s)->matchlist)
+    _gcmatch(&(*s)->matchlist);
+
   if((*s)->checksum)
     _gcchecksum(&(*s)->checksum);
 
@@ -435,6 +439,29 @@ static void _gcso(Size_T *s) {
 
 }
 
+static void _gcmatch(Match_T *s) {
+  
+  ASSERT(s);
+
+  if((*s)->next)
+    _gcmatch(&(*s)->next);
+
+  if((*s)->action)
+    _gc_eventaction(&(*s)->action);
+
+  if((*s)->match_path)
+      FREE((*s)->match_path);
+  if((*s)->match_string)
+      FREE((*s)->match_string);
+#ifdef HAVE_REGEX_H
+  if((*s)->regex_comp)
+      FREE((*s)->regex_comp);
+#endif
+  
+  FREE(*s);
+
+}
+
 
 static void _gcchecksum(Checksum_T *s) {
   
