@@ -111,6 +111,7 @@ static void print_service_rules_gid(HttpResponse, Service_T);
 static void print_service_rules_timestamp(HttpResponse, Service_T);
 static void print_service_rules_device(HttpResponse, Service_T);
 static void print_service_rules_size(HttpResponse, Service_T);
+static void print_service_rules_match(HttpResponse, Service_T);
 static void print_service_rules_checksum(HttpResponse, Service_T);
 static void print_service_rules_process(HttpResponse, Service_T);
 static void print_service_params_port(HttpResponse, Service_T);
@@ -121,6 +122,7 @@ static void print_service_params_gid(HttpResponse, Service_T);
 static void print_service_params_timestamp(HttpResponse, Service_T);
 static void print_service_params_device(HttpResponse, Service_T);
 static void print_service_params_size(HttpResponse, Service_T);
+static void print_service_params_match(HttpResponse, Service_T);
 static void print_service_params_checksum(HttpResponse, Service_T);
 static void print_service_params_process(HttpResponse, Service_T);
 static void print_status(HttpRequest, HttpResponse);
@@ -140,7 +142,7 @@ static char *get_service_status_text(Service_T);
  *  @author Martin Pala <martinp@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: cervlet.c,v 1.186 2005/04/05 19:52:57 martinp Exp $
+ *  @version \$Id: cervlet.c,v 1.191 2005/09/25 21:02:54 martinp Exp $
  *
  *  @file
  */
@@ -754,7 +756,7 @@ static void do_service(HttpRequest req, HttpResponse res, Service_T s) {
       EventAction_T a= s->action_TIMEOUT;
       out_print(res,
         "<tr><td>Timeout</td>"
-        "<td>If %d restart within %d cycles then %s else if recovered "
+        "<td>If %d restart within %d cycles then %s else if passed "
         "then %s</td></tr>",
          s->to_start, s->to_cycle, actionnames[a->failed->id],
          actionnames[a->passed->id]);
@@ -774,6 +776,7 @@ static void do_service(HttpRequest req, HttpResponse res, Service_T s) {
     print_service_params_timestamp(res, s);
     print_service_params_device(res, s);
     print_service_params_size(res, s);
+    print_service_params_match(res, s);
     print_service_params_checksum(res, s);
     print_service_params_process(res, s);
 
@@ -786,6 +789,7 @@ static void do_service(HttpRequest req, HttpResponse res, Service_T s) {
     print_service_rules_timestamp(res, s);
     print_service_rules_device(res, s);
     print_service_rules_size(res, s);
+    print_service_rules_match(res, s);
     print_service_rules_checksum(res, s);
     print_service_rules_process(res, s);
 
@@ -1297,6 +1301,8 @@ static void print_alerts(HttpResponse res, Mail_T s) {
 	  out_print(res, "Icmp ");
       if(IS_EVENT_SET(r->events, EVENT_INVALID))
 	  out_print(res, "Invalid ");
+      if(IS_EVENT_SET(r->events, EVENT_MATCH))
+	  out_print(res, "Match ");
       if(IS_EVENT_SET(r->events, EVENT_NONEXIST))
 	  out_print(res, "Nonexist ");
       if(IS_EVENT_SET(r->events, EVENT_PERMISSION))
@@ -1305,6 +1311,8 @@ static void print_alerts(HttpResponse res, Mail_T s) {
 	  out_print(res, "Resource ");
       if(IS_EVENT_SET(r->events, EVENT_SIZE))
 	  out_print(res, "Size ");
+      if(IS_EVENT_SET(r->events, EVENT_MATCH))
+	  out_print(res, "Match ");
       if(IS_EVENT_SET(r->events, EVENT_TIMEOUT))
 	  out_print(res, "Timeout ");
       if(IS_EVENT_SET(r->events, EVENT_TIMESTAMP))
@@ -1315,6 +1323,12 @@ static void print_alerts(HttpResponse res, Mail_T s) {
       
     out_print(res, "</td></tr>");
 
+    if(r->reminder) {
+       out_print(res,
+         "<tr><td>Alert reminder</td><td>%u cycles</td></tr>",
+         r->reminder);
+    }
+
   }
 
 }
@@ -1368,19 +1382,26 @@ static void print_service_rules_port(HttpResponse res, Service_T s) {
 
   if(s->portlist) {
 
+    char          ratio1[STRLEN];
+    char          ratio2[STRLEN];
     Port_T        p;
     EventAction_T a;
 
     for(p= s->portlist; p; p= p->next) {
       a= p->action;
+
+      Util_getEventratio(a->failed, ratio1);
+      Util_getEventratio(a->passed, ratio2);
+
       if(p->family == AF_INET) {
         out_print(res,
           "<tr><td>Port</td><td>"
-          "If failed %s:%d%s [%s%s] with timeout %d seconds then %s "
-          "else if recovered then %s</td></tr>",
+          "If failed %s:%d%s [%s%s] with timeout %d seconds %s then %s "
+          "else if passed %s then %s</td></tr>",
           p->hostname, p->port, p->request?p->request:"",
           p->SSL.use_ssl?" via SSL":"", p->protocol->name, p->timeout,
-          actionnames[a->failed->id], actionnames[a->passed->id]);
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
         if(p->SSL.certmd5 != NULL)
           out_print(res,
             "<tr><td>Server certificate md5 sum</td><td>%s</td></tr>",
@@ -1388,10 +1409,11 @@ static void print_service_rules_port(HttpResponse res, Service_T s) {
       } else if(p->family == AF_UNIX) {
         out_print(res,
           "<tr><td>Unix Socket</td><td>"
-          "If failed %s [%s] with timeout %ds then %s else if recovered then %s"
+          "If failed %s [%s] with timeout %ds %s then %s else if passed %s then %s"
           "</td></tr>",
           p->pathname, p->protocol->name, p->timeout,
-          actionnames[a->failed->id], actionnames[a->passed->id]);
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
       }
     }
   }
@@ -1402,17 +1424,24 @@ static void print_service_rules_icmp(HttpResponse res, Service_T s) {
 
   if(s->icmplist) {
 
+    char          ratio1[STRLEN];
+    char          ratio2[STRLEN];
     Icmp_T        i;
     EventAction_T a;
 
     for(i= s->icmplist; i; i= i->next) {
       a= i->action;
+      
+      Util_getEventratio(a->failed, ratio1);
+      Util_getEventratio(a->passed, ratio2);
+
       out_print(res,
         "<tr><td>ICMP</td><td>"
-        "If failed %s with timeout %d seconds then %s else if recovered then %s"
+        "If failed %s count %d with timeout %d seconds %s then %s else if passed %s then %s"
         "</td></tr>",
-        icmpnames[i->type], i->timeout, actionnames[a->failed->id],
-        actionnames[a->passed->id]);
+        icmpnames[i->type], i->count, i->timeout,
+        ratio1, actionnames[a->failed->id],
+        ratio2, actionnames[a->passed->id]);
     }
   }
 }
@@ -1422,11 +1451,18 @@ static void print_service_rules_perm(HttpResponse res, Service_T s) {
 
   if(s->perm) {
 
+    char          ratio1[STRLEN];
+    char          ratio2[STRLEN];
     EventAction_T a= s->perm->action;
 
+    Util_getEventratio(a->failed, ratio1);
+    Util_getEventratio(a->passed, ratio2);
+
     out_print(res, "<tr><td>Associated permission</td>"
-      "<td>If failed %o then %s else if recovered then %s</td></tr>",
-      s->perm->perm, actionnames[a->failed->id], actionnames[a->passed->id]);
+      "<td>If failed %o %s then %s else if passed %s then %s</td></tr>",
+      s->perm->perm,
+      ratio1, actionnames[a->failed->id],
+      ratio2, actionnames[a->passed->id]);
 
   }
 }
@@ -1436,11 +1472,18 @@ static void print_service_rules_uid(HttpResponse res, Service_T s) {
 
   if(s->uid) {
 
+    char          ratio1[STRLEN];
+    char          ratio2[STRLEN];
     EventAction_T a= s->uid->action;
 
+    Util_getEventratio(a->failed, ratio1);
+    Util_getEventratio(a->passed, ratio2);
+
     out_print(res, "<tr><td>Associated UID</td>"
-      "<td>If failed %d then %s else if recovered then %s</td></tr>",
-      (int)s->uid->uid, actionnames[a->failed->id], actionnames[a->passed->id]);
+      "<td>If failed %d %s then %s else if passed %s then %s</td></tr>",
+      (int)s->uid->uid,
+      ratio1, actionnames[a->failed->id],
+      ratio2, actionnames[a->passed->id]);
   }
 }
 
@@ -1449,11 +1492,18 @@ static void print_service_rules_gid(HttpResponse res, Service_T s) {
 
   if(s->gid) {
 
+    char          ratio1[STRLEN];
+    char          ratio2[STRLEN];
     EventAction_T a= s->gid->action;
 
+    Util_getEventratio(a->failed, ratio1);
+    Util_getEventratio(a->passed, ratio2);
+
     out_print(res, "<tr><td>Associated GID</td>"
-      "<td>If failed %d then %s else if recovered then %s</td></tr>",
-      (int)s->gid->gid, actionnames[a->failed->id], actionnames[a->passed->id]);
+      "<td>If failed %d %s then %s else if passed %s then %s</td></tr>",
+      (int)s->gid->gid,
+      ratio1, actionnames[a->failed->id],
+      ratio2, actionnames[a->passed->id]);
   }
 }
 
@@ -1462,6 +1512,8 @@ static void print_service_rules_timestamp(HttpResponse res, Service_T s) {
 
   if(s->timestamplist) {
 
+    char          ratio1[STRLEN];
+    char          ratio2[STRLEN];
     Timestamp_T   t;
     EventAction_T a;
 
@@ -1469,19 +1521,23 @@ static void print_service_rules_timestamp(HttpResponse res, Service_T s) {
 
       a= t->action;
 
+      Util_getEventratio(a->failed, ratio1);
+      Util_getEventratio(a->passed, ratio2);
+
       if(t->test_changes) {
 
         out_print(res,
-          "<tr><td>Associated timestamp</td><td>If changed then %s</td></tr>",
-          actionnames[a->failed->id]);
+          "<tr><td>Associated timestamp</td><td>If changed %s then %s</td></tr>",
+          ratio1, actionnames[a->failed->id]);
 
       } else {
 
         out_print(res,
           "<tr><td>Associated timestamp</td>"
-          "<td>If %s %d second(s) then %s else if recovered then %s</td></tr>",
+          "<td>If %s %d second(s) %s then %s else if passed %s then %s</td></tr>",
           operatornames[t->operator], t->time,
-          actionnames[a->failed->id], actionnames[a->passed->id]);
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
       }
     }
   }
@@ -1492,47 +1548,56 @@ static void print_service_rules_device(HttpResponse res, Service_T s) {
 
   if(s->devicelist) {
       
+    char          ratio1[STRLEN];
+    char          ratio2[STRLEN];
     Device_T      dl;
     EventAction_T a;
 
     for(dl= s->devicelist; dl; dl= dl->next) {
 
       a= dl->action;
-	
+
+      Util_getEventratio(a->failed, ratio1);
+      Util_getEventratio(a->passed, ratio2);
+
       if(dl->resource == RESOURCE_ID_INODE) {
 	  
         if(dl->limit_absolute > -1) {
           out_print(res,
-	    "<tr><td>Inodes usage limit</td><td>If %s %ld "
-	    "then %s else if recovered then %s</td></tr>",
+	    "<tr><td>Inodes usage limit</td><td>If %s %ld %s "
+	    "then %s else if passed %s then %s</td></tr>",
 	    operatornames[dl->operator],
 	    dl->limit_absolute,
-	    actionnames[a->failed->id], actionnames[a->passed->id]);
+            ratio1, actionnames[a->failed->id],
+            ratio2, actionnames[a->passed->id]);
         } else {
           out_print(res,
-	    "<tr><td>Inodes usage limit</td><td>If %s %.1f%% "
-	    "then %s else if recovered then %s</td></tr>",
+	    "<tr><td>Inodes usage limit</td><td>If %s %.1f%% %s "
+	    "then %s else if passed %s then %s</td></tr>",
 	    operatornames[dl->operator],
 	    dl->limit_percent/10.,
-	    actionnames[a->failed->id], actionnames[a->passed->id]);
+            ratio1, actionnames[a->failed->id],
+            ratio2, actionnames[a->passed->id]);
         }
 	  
       } else if(dl->resource == RESOURCE_ID_SPACE) {
 	  
         if(dl->limit_absolute > -1) {
           out_print(res,
-	    "<tr><td>Space usage limit</td><td>If %s %ld blocks "
-	    "then %s else if recovered then %s</td></tr>",
+	    "<tr><td>Space usage limit</td><td>If %s %ld blocks %s "
+	    "then %s else if passed %s then %s</td></tr>",
             operatornames[dl->operator],
 	    dl->limit_absolute,
-	    actionnames[a->failed->id], actionnames[a->passed->id]);
+            ratio1, actionnames[a->failed->id],
+            ratio2, actionnames[a->passed->id]);
         } else {
           out_print(res,
-	    "<tr><td>Space usage limit</td><td>If %s %.1f%% "
-	    "then %s else if recovered then %s</td></tr>",
+	    "<tr><td>Space usage limit</td><td>If %s %.1f%% %s "
+	    "then %s else if passed %s then %s</td></tr>",
             operatornames[dl->operator],
 	    dl->limit_percent/10.,
-	    actionnames[a->failed->id], actionnames[a->passed->id]);
+            ratio1, actionnames[a->failed->id],
+            ratio2, actionnames[a->passed->id]);
         }
       }
     }
@@ -1544,6 +1609,8 @@ static void print_service_rules_size(HttpResponse res, Service_T s) {
 
   if(s->sizelist) {
 
+    char          ratio1[STRLEN];
+    char          ratio2[STRLEN];
     Size_T        sl;
     EventAction_T a;
 
@@ -1551,45 +1618,80 @@ static void print_service_rules_size(HttpResponse res, Service_T s) {
 
       a= sl->action;
 
+      Util_getEventratio(a->failed, ratio1);
+      Util_getEventratio(a->passed, ratio2);
+
       if(sl->test_changes) {
 
         out_print(res,
-          "<tr><td>Associated size</td><td>If changed then %s</td></tr>",
-          actionnames[a->failed->id]);
+          "<tr><td>Associated size</td><td>If changed %s then %s</td></tr>",
+          ratio1, actionnames[a->failed->id]);
 
       } else {
 
         out_print(res,
           "<tr><td>Associated size</td>"
-          "<td>If %s %lu byte(s) then %s else if recovered then %s</td></tr>",
+          "<td>If %s %lu byte(s) %s then %s else if passed %s then %s</td></tr>",
           operatornames[sl->operator], sl->size,
-          actionnames[a->failed->id], actionnames[a->passed->id]);
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
 
       }
     }
   }
 }
 
+static void print_service_rules_match(HttpResponse res, Service_T s) {
+
+  if(s->matchlist) {
+
+    char          ratio1[STRLEN];
+    Match_T       ml;
+    EventAction_T a;
+
+    for(ml= s->matchlist; ml; ml= ml->next) {
+
+      a= ml->action;
+
+      Util_getEventratio(a->failed, ratio1);
+
+      out_print(res,
+                "<tr><td>Associated regex</td><td>If %s match "
+                "\"%s\" %s then %s</td></tr>",
+                ml->not?"not ":"", ml->match_string,
+                ratio1, actionnames[a->failed->id]);
+      
+    }
+  }
+}
+
 
 static void print_service_rules_checksum(HttpResponse res, Service_T s) {
 
   if(s->checksum) {
 
-    Checksum_T    cs= s->checksum;
+    char          ratio1[STRLEN];
+    char          ratio2[STRLEN];
+    Checksum_T     cs= s->checksum;
     EventAction_T  a= cs->action;
 
+    Util_getEventratio(a->failed, ratio1);
+    Util_getEventratio(a->passed, ratio2);
+
     if(cs->test_changes) {
 
       out_print(res,
-        "<tr><td>Associated checksum</td><td>if changed %s then %s</td></tr>",
-        checksumnames[cs->type], actionnames[a->failed->id]);
+        "<tr><td>Associated checksum</td><td>if changed %s %s then %s</td></tr>",
+        checksumnames[cs->type],
+        ratio1, actionnames[a->failed->id]);
 
     } else {
 
       out_print(res, "<tr><td>Associated checksum</td><td>"
-        "if failed %s(%s) then %s else if recovered then %s</td></tr>",
+        "if failed %s(%s) %s then %s else if passed %s then %s</td></tr>",
         cs->hash, checksumnames[cs->type],
-        actionnames[a->failed->id], actionnames[a->passed->id]);
+        ratio1, actionnames[a->failed->id],
+        ratio2, actionnames[a->passed->id]);
 
     }
   }
@@ -1598,10 +1700,15 @@ static void print_service_rules_checksum(HttpResponse res, Service_T s) {
 
 static void print_service_rules_process(HttpResponse res, Service_T s) {
 
+  char ratio1[STRLEN];
+  char ratio2[STRLEN];
+
   if(s->type == TYPE_PROCESS) {
-    out_print(res, "<tr><td>Pid</td><td>If changed then %s</td></tr>\n",
+    out_print(res, "<tr><td>Pid</td><td>If changed %s then %s</td></tr>\n",
+      Util_getEventratio(s->action_PID->failed, ratio1),
       actionnames[s->action_PID->failed->id]);
-    out_print(res, "<tr><td>Ppid</td><td>If changed then %s</td></tr>\n",
+    out_print(res, "<tr><td>Ppid</td><td>If changed %s then %s</td></tr>\n",
+      Util_getEventratio(s->action_PPID->failed, ratio1),
       actionnames[s->action_PPID->failed->id]);
   }
 
@@ -1614,96 +1721,108 @@ static void print_service_rules_process(HttpResponse res, Service_T s) {
 
       a= q->action;
 
+      Util_getEventratio(a->failed, ratio1);
+      Util_getEventratio(a->passed, ratio2);
+
       switch (q->resource_id) {
 	
       case RESOURCE_ID_CPU_PERCENT: 
 	  
 	out_print(res,"<tr><td>CPU usage limit</td>"
-	  "<td>If %s %.1f%% for %d cycle(s) then %s "
-          "else if recovered then %s</td></tr>", 
+	  "<td>If %s %.1f%% %s then %s "
+          "else if passed %s then %s</td></tr>", 
 	  operatornames[q->operator],
-	  q->limit/10.0, q->max_cycle, 
-	  actionnames[a->failed->id], actionnames[a->passed->id]);
+	  q->limit/10.0,
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
 	break;
 	  
       case RESOURCE_ID_MEM_PERCENT: 
 	  
 	out_print(res,"<tr><td>Memory usage limit</td>"
-	  "<td>If %s %.1f%% for %d cycle(s) then %s "
-          "else if recovered then %s</td></tr>", 
+	  "<td>If %s %.1f%% %s then %s "
+          "else if passed %s then %s</td></tr>", 
 	  operatornames[q->operator],
-	  q->limit/10.0, q->max_cycle, 
-	  actionnames[a->failed->id], actionnames[a->passed->id]);
+	  q->limit/10.0,
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
 	break;
 	  
       case RESOURCE_ID_MEM_KBYTE: 
 	  
 	out_print(res,"<tr><td>Memory amount limit</td>"
-	  "<td>If %s %ld for %d cycle(s) then %s "
-          "else if recovered then %s</td></tr>", 
+	  "<td>If %s %ld %s then %s "
+          "else if passed %s then %s</td></tr>", 
 	  operatornames[q->operator],
-	  q->limit, q->max_cycle, 
-	  actionnames[a->failed->id], actionnames[a->passed->id]);
+	  q->limit,
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
 	break;
 	  
       case RESOURCE_ID_LOAD1: 
 	  
 	out_print(res,"<tr><td>Load average (1min)</td>"
-	  "<td>If %s %.1f for %d cycle(s) then %s "
-          "else if recovered then %s</td></tr>", 
+	  "<td>If %s %.1f %s then %s "
+          "else if passed %s then %s</td></tr>", 
 	  operatornames[q->operator],
-	  q->limit/10.0, q->max_cycle, 
-	  actionnames[a->failed->id], actionnames[a->passed->id]);
+	  q->limit/10.0,
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
 	break;
 	  
       case RESOURCE_ID_LOAD5: 
 	  
 	out_print(res,"<tr><td>Load average (5min)</td>"
-	  "<td>If %s %.1f for %d cycle(s) then %s else if recovered "
+	  "<td>If %s %.1f %s then %s else if passed %s "
           "then %s</td></tr>", 
 	  operatornames[q->operator],
-	  q->limit/10.0, q->max_cycle, 
-	  actionnames[a->failed->id], actionnames[a->passed->id]);
+	  q->limit/10.0,
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
 	break;
 	  
       case RESOURCE_ID_LOAD15: 
 
 	out_print(res,"<tr><td>Load average (15min)</td>"
-	  "<td>If %s %.1f for %d cycle(s) then %s else if recovered "
+	  "<td>If %s %.1f %s then %s else if passed %s "
           "then %s</td></tr>", 
 	  operatornames[q->operator],
-	  q->limit/10.0, q->max_cycle, 
-	  actionnames[a->failed->id], actionnames[a->passed->id]);
+	  q->limit/10.0,
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
 	break;
 	  
       case RESOURCE_ID_CHILDREN:
 
 	out_print(res,"<tr><td>Children</td>"
-	  "<td>If %s %d for %d cycle(s) then %s else if recovered "
+	  "<td>If %s %d %s then %s else if passed %s "
           "then %s</td></tr>",
 	  operatornames[q->operator],
-	  q->limit, q->max_cycle,
-	  actionnames[a->failed->id], actionnames[a->passed->id]);
+	  q->limit,
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
 	break;
 	  
       case RESOURCE_ID_TOTAL_MEM_KBYTE:
 
 	out_print(res,"<tr><td>Memory amount limit (incl. children)</td>"
-	  "<td>If %s %d for %d cycle(s) then %s else if recovered then %s"
+	  "<td>If %s %d %s then %s else if passed %s then %s"
           "</td></tr>",
 	  operatornames[q->operator],
-	  q->limit, q->max_cycle,
-	  actionnames[a->failed->id], actionnames[a->passed->id]);
+	  q->limit,
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
 	break;
 	  
       case RESOURCE_ID_TOTAL_MEM_PERCENT:
 
 	out_print(res,"<tr><td>Memory usage limit (incl. children)</td>"
-	  "<td>If %s %.1f%% for %d cycle(s) then %s else if recovered then %s"
+	  "<td>If %s %.1f%% %s then %s else if passed %s then %s"
           "</td></tr>",
 	  operatornames[q->operator],
-	  q->limit/10.0, q->max_cycle,
-	  actionnames[a->failed->id], actionnames[a->passed->id]);
+	  q->limit/10.0,
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
 	break;
 	  
       }    
@@ -1963,6 +2082,25 @@ static void print_service_params_size(HttpResponse res, Service_T s) {
   }
 }
 
+static void print_service_params_match(HttpResponse res, Service_T s) {
+
+  if(s->type == TYPE_FILE) {
+
+    if(!Util_hasServiceStatus(s)) {
+
+      out_print(res,
+        "<tr><td>Match regex</td><td>-</font></td></tr>");
+
+    } else {
+
+      out_print(res,
+        "<tr><td>Match regex</td><td><font%s>%s</td></tr>",
+        (s->error & EVENT_MATCH)?" color='#ff0000'":"",
+        (s->error & EVENT_MATCH)?"yes":"no");
+    }
+  }
+}
+
 
 static void print_service_params_checksum(HttpResponse res, Service_T s) {
 
