@@ -107,7 +107,7 @@
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *  @author Martin Pala <martinp@tildeslash.com>
  *
- *  @version \$Id: engine.c,v 1.68 2005/02/12 11:39:43 martinp Exp $
+ *  @version \$Id: engine.c,v 1.70 2005/08/05 09:38:10 chopp Exp $
  *
  *  @file 
  */
@@ -307,7 +307,7 @@ int add_host_allow(char *name) {
 
 int add_net_allow(char *s_network) {
 
-  struct ulong_net net;
+  struct ulong_net net={0, 0};
   HostsAllow h;
 
   ASSERT(s_network);
@@ -601,7 +601,7 @@ static int parse_network(char *s_network, struct ulong_net *net) {
     } else {
       
       net->mask= (1<<shortmask)-1;
-      net->mask= (net->mask<<(32-shortmask));
+      net->mask= htonl(net->mask<<(32-shortmask));
 
     }
 
@@ -642,7 +642,7 @@ static Socket_T socket_producer(int server, int port, void *sslserver) {
   
   int client;
   struct sockaddr_in in;
-  int len= sizeof(struct sockaddr_in);
+  unsigned int len= sizeof(struct sockaddr_in);
   
   if(can_read(server, 1)) {
     
