@@ -80,7 +80,7 @@
  *  @author Martin Pala <martinp@tildeslash.com>
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: monitor.c,v 1.120 2005/04/05 19:52:57 martinp Exp $
+ *  @version \$Id: monitor.c,v 1.121 2005/09/11 14:57:02 martinp Exp $
  *
  *  @file
  */
@@ -374,7 +374,9 @@ static void do_action(char **args) {
 
       }
     } else {
-      goto error;
+      log("%s: please specify the configured service name or 'all' after %s\n",
+        prog, action);
+      exit(1);
     }
   } else if(IS(action, "reload")) {
     log("Reinitializing monit daemon\n", prog);
@@ -388,7 +390,6 @@ static void do_action(char **args) {
   } else if(IS(action, "validate")) {
     validate();
   } else {
-  error:
     log("%s: invalid argument -- %s  (-h will show valid arguments)\n",
         prog, action);
     exit(1);
