@@ -69,7 +69,7 @@
 #define STRERROR            strerror(errno)
 #define STRLEN             256
 #define ARGMAX             64
-#define HTTP_CONTENT_MAX   (1024*10)
+#define HTTP_CONTENT_MAX   (1024*1000)
 /* Set log file mode: "-rw-rw-r--" */
 #define LOGMASK            0112
 /* Set pid file mode: "-rw-r--r--" */
@@ -96,6 +96,10 @@
 #define MONITOR_YES        1
 #define MONITOR_INIT       2
 
+#define STATE_PASSED       0
+#define STATE_FAILED       1
+#define STATE_INIT         2
+
 #define MODE_ACTIVE        0
 #define MODE_PASSIVE       1
 #define MODE_MANUAL        2
@@ -161,6 +165,8 @@
 #define LEVEL_NAME_FULL    "full"
 #define LEVEL_NAME_SUMMARY "summary"
 
+#define ATTEMPT_COUNT      3
+
 /** ------------------------------------------------- Special purpose macros */
 
 
@@ -234,6 +240,8 @@ typedef struct mycommand {
 typedef struct myaction {
     int       id;                                     /**< Action to be done */
     Command_T exec;                    /**< Optional command to be executed  */
+    unsigned  count;           /**< Event count needed to trigger the action */
+    unsigned  cycles;    /**< Cycles during which count limit can be reached */
 } *Action_T;
 
 
@@ -289,6 +297,7 @@ typedef struct mymail {
   char *message;                                       /**< The mail message */
   char *opt_message;                 /**< An optional message used in alerts */
   unsigned int events;  /*< Events for which this mail object should be sent */
+  unsigned int reminder;              /*< Send error reminder each Xth cycle */
 
   /** For internal use */
   struct mymail *next;                          /**< next recipient in chain */
@@ -480,6 +489,7 @@ typedef struct myport {
 /** Defines a ICMP object */
 typedef struct myicmp {
   int type;                                              /**< ICMP type used */
+  int count;                                   /**< ICMP echo requests count */
   int timeout;              /**< The timeout in seconds to wait for response */
   int is_available;                     /**< TRUE if the server is available */
   double response;                              /**< ICMP ECHO response time */
@@ -503,8 +513,6 @@ typedef struct myresource {
   int  resource_id;                              /**< Which value is checked */
   long limit;                                     /**< Limit of the resource */
   int  operator;                                    /**< Comparison operator */
-  int  cycle;                                     /**< Cycle overrun counter */
-  int  max_cycle;                                   /**< Cycle overrun limit */
   EventAction_T action;  /**< Description of the action upon event occurence */
   
   /** For internal use */
@@ -553,6 +561,21 @@ typedef struct myperm {
   EventAction_T action;  /**< Description of the action upon event occurence */
 } *Perm_T;
 
+/** Defines match object */
+typedef struct mymatch {
+  int     ignore;                                          /**< Ignore match */
+  int     not;                                             /**< Invert match */
+  char    *match_string;                                   /**< Match string */
+  char    *match_path;                         /**< File with matching rules */
+#ifdef HAVE_REGEX_H
+  regex_t *regex_comp;                                    /**< Match compile */
+#endif
+  EventAction_T action;  /**< Description of the action upon event occurence */
+ 
+  /** For internal use */
+  struct mymatch *next;                             /**< next match in chain */
+} *Match_T;
+
 
 /** Defines uid object */
 typedef struct myuid {
@@ -588,6 +611,7 @@ typedef struct myinfo {
   mode_t  st_mode;                                           /**< Permission */
   uid_t   st_uid;                                           /**< Owner's uid */
   gid_t   st_gid;                                           /**< Owner's gid */
+  ino_t   st_ino;                                                 /**< Inode */
   time_t  timestamp;                                          /**< Timestamp */
 
   /* Device specific */
@@ -605,6 +629,8 @@ typedef struct myinfo {
 
   /* File specific */
   size_t st_size;                                                  /**< Size */
+  size_t readpos;                           /**< Position for regex matching */
+  ino_t  st_ino_prev;                 /**< Previous inode for regex matching */
   char  *cs_sum;                                               /**< Checksum */
 
   /* Process specific */
@@ -621,7 +647,6 @@ typedef struct myinfo {
   int    cpu_percent;                                    /**< pecentage * 10 */
   int    total_cpu_percent;                              /**< pecentage * 10 */
   time_t uptime;                                         /**< Process uptime */
-
 } *Info_T;
 
 
@@ -660,6 +685,7 @@ typedef struct myservice {
   Port_T      portlist; /**< Portnumbers to check, either local or at a host */
   Resource_T  resourcelist;                          /**< Resouce check list */
   Size_T      sizelist;                                 /**< Size check list */
+  Match_T     matchlist;                             /**< Content Match list */
   Timestamp_T timestamplist;                       /**< Timestamp check list */
   Uid_T       uid;                                            /**< Uid check */
   
@@ -686,6 +712,8 @@ typedef struct myservice {
     struct myservice *source;                              /**< Event source */
     short             state;            /**< TRUE if failed, FALSE if passed */
     short             state_changed;              /**< TRUE if state changed */
+    long long         state_map;           /**< Event bitmap for last cycles */
+    unsigned int      count;                             /**< The event rate */
     char             *message;    /**< Optional message describing the event */
     EventAction_T     action;           /**< Description of the event action */
     /** For internal use */
