@@ -134,7 +134,7 @@
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: net.c,v 1.59 2005/01/25 05:34:58 hauk Exp $
+ *  @version \$Id: net.c,v 1.62 2005/10/05 21:37:47 martinp Exp $
  *
  *  @file
  */
@@ -579,11 +579,13 @@ int sock_read(int socket, void *buffer, int size, int timeout) {
 
 /**
  * Create a ICMP socket against hostname, send echo and wait for response.
+ * The 'count' echo requests  is send and we expect at least one reply.
  * @param hostname The host to open a socket at
  * @param timeout If response will not come within timeout seconds abort
+ * @param count How many pings to send
  * @return response time on succes, -1 on error
  */
-double icmp_echo(const char *hostname, int timeout) {
+double icmp_echo(const char *hostname, int timeout, int count) {
 
   struct hostent *hp;
   struct sockaddr_in sin;
@@ -591,21 +593,22 @@ double icmp_echo(const char *hostname, int timeout) {
 #ifdef HAVE_SOL_IP
   struct iphdr *iphdrin;
   struct icmphdr *icmphdrin= NULL;
-  struct icmphdr *icmphdrout= NULL;
+  struct icmphdr *icmphdrout[count];
 #else
   struct ip *iphdrin;
   struct icmp *icmphdrin= NULL;
-  struct icmp *icmphdrout= NULL;
+  struct icmp *icmphdrout[count];
 #endif
   size_t size;
   fd_set rset;
+  int i;
   int s;
   int n= 0;
   int sol_ip;
   unsigned ttl= 255;
   char buf[STRLEN];
   struct timeval tv;
-  struct timeval t1;
+  struct timeval t1[count];
   struct timeval t2;
   double response= -1;
   
@@ -635,31 +638,33 @@ double icmp_echo(const char *hostname, int timeout) {
   tv.tv_sec= timeout;
   tv.tv_usec= 0;
 
-  NEW(icmphdrout);
+  for(i=0; i<count; i++) {
+    NEW(icmphdrout[i]);
 #ifdef HAVE_SOL_IP
-  icmphdrout->code= 0;
-  icmphdrout->type= ICMP_ECHO;
-  icmphdrout->un.echo.id= getpid();
-  icmphdrout->un.echo.sequence= 0;
-  icmphdrout->checksum= checksum_ip((unsigned char *)icmphdrout, ICMP_SIZE);
+    icmphdrout[i]->code= 0;
+    icmphdrout[i]->type= ICMP_ECHO;
+    icmphdrout[i]->un.echo.id= getpid() + (int)hostname + time(NULL);
+    icmphdrout[i]->un.echo.sequence= i;
+    icmphdrout[i]->checksum= checksum_ip((unsigned char *)icmphdrout[i], ICMP_SIZE);
 #else
-  icmphdrout->icmp_code= 0;
-  icmphdrout->icmp_type= ICMP_ECHO;
-  icmphdrout->icmp_id= getpid();
-  icmphdrout->icmp_seq= 0;
-  icmphdrout->icmp_cksum= checksum_ip((unsigned char *)icmphdrout, ICMP_SIZE);
+    icmphdrout[i]->icmp_code= 0;
+    icmphdrout[i]->icmp_type= ICMP_ECHO;
+    icmphdrout[i]->icmp_id= getpid() + (int)hostname + time(NULL);
+    icmphdrout[i]->icmp_seq= i;
+    icmphdrout[i]->icmp_cksum= checksum_ip((unsigned char *)icmphdrout[i], ICMP_SIZE);
 #endif
-  sout.sin_family= AF_INET;
-  sout.sin_port= 0;
-  memcpy(&sout.sin_addr, hp->h_addr, hp->h_length);
+    sout.sin_family= AF_INET;
+    sout.sin_port= 0;
+    memcpy(&sout.sin_addr, hp->h_addr, hp->h_length);
 
-  /* Get time of connection attempt beginning */
-  gettimeofday(&t1, NULL);
+    /* Get time of particular connection attempt beginning */
+    gettimeofday(&t1[i], NULL);
 
-  do {
-    n= sendto(s, (char *)icmphdrout, ICMP_SIZE, 0,
+    do {
+      n= sendto(s, (char *)icmphdrout[i], ICMP_SIZE, 0,
 	      (struct sockaddr *)&sout, sizeof(struct sockaddr));
-  } while(n == -1 && errno == EINTR);
+    } while(n == -1 && errno == EINTR);
+  }
   
   do {
 
@@ -680,36 +685,40 @@ double icmp_echo(const char *hostname, int timeout) {
     } while(n == -1 && errno == EINTR);
     
     if(n < 0)
-	goto error;
+	    goto error;
     
+    for(i=0; i<count; i++) {
 #ifdef HAVE_SOL_IP
-    iphdrin= (struct iphdr *)buf;
-    icmphdrin= (struct icmphdr *)(buf + iphdrin->ihl * 4);
-    if( (icmphdrin->un.echo.id == icmphdrout->un.echo.id) &&
-        (icmphdrin->type == ICMP_ECHOREPLY) &&
-        (icmphdrin->un.echo.sequence == icmphdrout->un.echo.sequence) ) {
+      iphdrin= (struct iphdr *)buf;
+      icmphdrin= (struct icmphdr *)(buf + iphdrin->ihl * 4);
+      if( (icmphdrin->un.echo.id == icmphdrout[i]->un.echo.id) &&
+          (icmphdrin->type == ICMP_ECHOREPLY) &&
+          (icmphdrin->un.echo.sequence == icmphdrout[i]->un.echo.sequence) ) {
 #else
-    iphdrin= (struct ip *)buf;
-    icmphdrin= (struct icmp *)(buf + iphdrin->ip_hl * 4);
-    if( (icmphdrin->icmp_id == icmphdrout->icmp_id) &&
-        (icmphdrin->icmp_type == ICMP_ECHOREPLY) &&
-        (icmphdrin->icmp_seq == icmphdrout->icmp_seq) ) {
+      iphdrin= (struct ip *)buf;
+      icmphdrin= (struct icmp *)(buf + iphdrin->ip_hl * 4);
+      if( (icmphdrin->icmp_id == icmphdrout[i]->icmp_id) &&
+          (icmphdrin->icmp_type == ICMP_ECHOREPLY) &&
+          (icmphdrin->icmp_seq == icmphdrout[i]->icmp_seq) ) {
 #endif
 
-      /* Get time of connection attempt finish */
-      gettimeofday(&t2, NULL);
+        /* Get time of connection attempt finish */
+        gettimeofday(&t2, NULL);
 
-      /* Get the response time */
-      response= (double)(t2.tv_sec  - t1.tv_sec) +
-                (double)(t2.tv_usec - t1.tv_usec)/1000000;
-      break;
+        /* Get the response time */
+        response= (double)(t2.tv_sec  - t1[i].tv_sec) +
+                  (double)(t2.tv_usec - t1[i].tv_usec)/1000000;
 
+        goto done;
+      }
     }
-
   } while(TRUE);
 
   error:
-  FREE(icmphdrout);
+  done:
+  for(i=0; i<count; i++) {
+    FREE(icmphdrout[i]);
+  }
   close_socket(s);
 
   return response;
@@ -750,7 +759,7 @@ static int do_connect(int s, const struct sockaddr *addr,
   }
 
   if(FD_ISSET(s, &rset) || FD_ISSET(s, &wset)) {
-    int len= sizeof(error);
+    unsigned int len= sizeof(error);
     if(getsockopt(s, SOL_SOCKET, SO_ERROR, &error, &len) < 0)
       return -1;
   } else
