@@ -32,7 +32,7 @@
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: net.h,v 1.25 2005/01/06 20:51:49 martinp Exp $
+ *  @version \$Id: net.h,v 1.26 2005/08/05 11:21:18 martinp Exp $
  *
  *  @file
  */
@@ -190,10 +190,12 @@ int sock_read(int socket, void *buffer, int size, int timeout);
 
 /**
  * Create a ICMP socket against hostname, send echo and wait for response.
+ * The 'count' echo requests  is send and we expect at least one reply.
  * @param hostname The host to open a socket at
  * @param timeout If response will not come within timeout seconds abort
+ * @param count How many pings to send
  * @return response time on succes, -1 on error
  */
-double icmp_echo(const char *hostname, int timeout);
+double icmp_echo(const char *hostname, int timeout, int count);
 
 #endif
