@@ -61,7 +61,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: process.c,v 1.23 2005/04/10 22:21:59 martinp Exp $
+ *  @version \$Id: process.c,v 1.24 2005/04/19 21:02:56 martinp Exp $
  *
  *  @file
  */
@@ -287,7 +287,9 @@ int initprocesstree(ProcessTree_T **pt_r,     int *size_r,
   
   if((*size_r = initprocesstree_sysdep(pt_r)) <= 0)
   {
-    DEBUG("system statistic error -- cannot initialize the process tree\n");
+    DEBUG("system statistic error -- cannot initialize the process tree => "
+          "process resource monitoring disabled\n");
+    Run.doprocess = FALSE;
     return -1;
   }
 
