@@ -32,16 +32,16 @@
 #include <fcntl.h>
 #endif
 
-#ifdef HAVE_SYS_VMMETER_H
-#include <sys/vmmeter.h>
+#ifdef HAVE_KVM_H
+#include <kvm.h>
 #endif
 
 #ifdef HAVE_SYS_SYSCTL_H
 #include <sys/sysctl.h>
 #endif
 
-#ifdef HAVE_KVM_H
-#include <kvm.h>
+#ifdef HAVE_SYS_VMMETER_H
+#include <sys/vmmeter.h>
 #endif
 
 #ifdef HAVE_MACH_HOST_INFO_H
@@ -52,6 +52,7 @@
 #include <mach/mach_host.h>
 #endif
 
+
 #include "monitor.h"
 #include "process.h"
 #include "process_sysdep.h"
@@ -64,7 +65,7 @@
  *  @author Rory Toma <rory@digeo.com>
  *  @author Martin Pala <martinp@tildeslash.com>
  *
- *  @version \$Id: sysdep_DARWIN.c,v 1.22 2005/01/06 20:51:50 martinp Exp $
+ *  @version \$Id: sysdep_DARWIN.c,v 1.24 2005/08/02 23:14:17 hauk Exp $
  *
  *  @file
  */
@@ -144,13 +145,13 @@ int initprocesstree_sysdep(ProcessTree_T **reference) {
 
   if(getuid()!=0) {
     log("system statistic error -- permission denied\n");
-    goto error;
+    return FALSE;
   }
 
   if(!(kvm_handle = kvm_open(NULL, NULL, NULL, O_RDONLY, NULL)))
   {
     log("system statistic error -- cannot initialize kvm interface\n");
-    goto error;
+    return FALSE;
   }
 
   pinfo = kvm_getprocs(kvm_handle, KERN_PROC_ALL, 0, &treesize);
@@ -158,7 +159,7 @@ int initprocesstree_sysdep(ProcessTree_T **reference) {
   {
     log("system statistic error -- cannot get process tree\n");
     kvm_close(kvm_handle);
-    goto error;
+    return FALSE;
   }
 
   pt = xcalloc(sizeof(ProcessTree_T), treesize);
@@ -182,11 +183,6 @@ int initprocesstree_sysdep(ProcessTree_T **reference) {
   kvm_close(kvm_handle);
 
   return treesize;
-
-  error:
-  log("system statistic disabled\n");
-  Run.doprocess = FALSE;
-  return FALSE;
 }
 
 
