@@ -72,7 +72,7 @@
  *  @author Rory Toma <rory@digeo.com>
  *  @author Martin Pala<martinp@tildeslash.com>
  *
- *  @version \$Id: sysdep_FREEBSD.c,v 1.28 2005/01/06 20:51:50 martinp Exp $
+ *  @version \$Id: sysdep_FREEBSD.c,v 1.30 2005/04/19 21:47:28 martinp Exp $
  *
  *  @file
  */
@@ -152,7 +152,7 @@ int initprocesstree_sysdep(ProcessTree_T **reference) {
 
   if(getuid()!=0) {
     log("system statistic error -- permission denied\n");
-    goto error;
+    return FALSE;
   }
 
   if(!(kvm_handle = kvm_open(NULL, NULL, NULL, O_RDONLY, NULL)))
@@ -196,11 +196,6 @@ int initprocesstree_sysdep(ProcessTree_T **reference) {
   kvm_close(kvm_handle);
 
   return treesize;
-
-  error:
-  log("system statistic disabled\n");
-  Run.doprocess = FALSE;
-  return FALSE;
 }
 
 
