@@ -69,7 +69,7 @@
  *  @author Rory Toma <rory@digeo.com>
  *  @author Martin Pala<martinp@tildeslash.com>
  *
- *  @version \$Id: sysdep_OPENBSD.c,v 1.24 2005/01/06 20:51:50 martinp Exp $
+ *  @version \$Id: sysdep_OPENBSD.c,v 1.25 2005/04/19 21:47:28 martinp Exp $
  *
  *  @file
  */
@@ -149,13 +149,13 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
 
   if(getuid()!=0) {
     log("system statistic error -- permission denied\n");
-    goto error;
+    return FALSE;
   }
 
   if(!(kvm_handle = kvm_open(NULL, NULL, NULL, O_RDONLY, NULL)))
   {
     log("system statistic error -- cannot initialize kvm interface\n");
-    goto error;
+    return FALSE;
   }
 
   pinfo = kvm_getprocs(kvm_handle, KERN_PROC_ALL, 0, &treesize);
@@ -163,7 +163,7 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
   {
     log("system statistic error -- cannot get process tree\n");
     kvm_close(kvm_handle);
-    goto error;
+    return FALSE;
   }
 
   pt = xcalloc(sizeof(ProcessTree_T), treesize);
@@ -187,11 +187,6 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
   kvm_close(kvm_handle);
 
   return treesize;
-
-  error:
-  log("system statistic disabled\n");
-  Run.doprocess = FALSE;
-  return FALSE;
 }
 
 
