@@ -40,7 +40,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Michael Amster, <mamster@webeasy.com>
  *
- *  @version \$Id: ftp.c,v 1.18 2005/01/06 20:51:50 martinp Exp $
+ *  @version \$Id: ftp.c,v 1.20 2005/09/10 12:47:51 hauk Exp $
  *
  *  @file
  */
@@ -48,7 +48,6 @@ int check_ftp(Socket_T s) {
 
   int status;
   char buf[STRLEN];
-  char msg[STRLEN];
 
   ASSERT(s);
 
@@ -59,7 +58,7 @@ int check_ftp(Socket_T s) {
 
   Util_chomp(buf);
   
-  sscanf(buf, "%d %s", &status, msg);
+  sscanf(buf, "%d %*s", &status);
   if(status != 220) {
     log("FTP error: %s\n", buf);
     return FALSE;
@@ -67,7 +66,7 @@ int check_ftp(Socket_T s) {
 
   /* Read past banners */
   while(NULL != socket_readln(s, buf, STRLEN)) {
-    if(Util_startsWith(buf, "220")) continue;
+    if(!Util_startsWith(buf, "220")) break;
   }
   
   if(socket_print(s, "QUIT\r\n") < 0) {
@@ -82,7 +81,7 @@ int check_ftp(Socket_T s) {
 
   Util_chomp(buf);
 
-  sscanf(buf, "%d %s", &status, msg);
+  sscanf(buf, "%d %*s", &status);
   if(status != 221) {
     log("FTP error: %s\n", buf);
     return FALSE;
