@@ -73,7 +73,7 @@
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
- *  @version \$Id: http.c,v 1.42 2005/03/23 22:57:34 hauk Exp $
+ *  @version \$Id: http.c,v 1.47 2005/08/05 09:38:10 chopp Exp $
  *  @file
  */
 
@@ -274,8 +274,10 @@ static int check_request_checksum(Http_T *H, char *checksum, int hashtype) {
 static int do_regex(Http_T *H, Request_T R) {
 
   int n;
+  int size= 0;
   int rv= TRUE;
-  unsigned char *buf= NULL;
+  int length= 0;
+  char *buf= NULL;
 #ifdef HAVE_REGEX_H
   int regex_return;
 #else
@@ -295,16 +297,26 @@ static int do_regex(Http_T *H, Request_T R) {
     H->content_length= HTTP_CONTENT_MAX;
   else if(H->content_length > HTTP_CONTENT_MAX)
     H->content_length= HTTP_CONTENT_MAX;
-    
-  buf= xmalloc(H->content_length);
-
-  n= socket_read(H->s, buf, H->content_length);
-  if(n<=0) {
+  
+  n= 0;
+  size= 0;
+  length= H->content_length;
+  buf= xmalloc(H->content_length + 1);
+  
+  do {
+      n= socket_read(H->s, &buf[size], length);
+      if(n<=0)
+        break;
+      size+= n;
+      length-= n;
+  } while(length>0);
+  
+  if(size==0) {
     rv= FALSE;
     log("HTTP: error receiving data -- %s\n", STRERROR);
     goto error;
   }
-  buf[n]= 0;
+  buf[size]= 0;
 
 #ifdef HAVE_REGEX_H
 
@@ -373,12 +385,24 @@ error:
 /* -------------------------------------------------------- State management */
 
 
+/*
+ * Follow redirect responses from the server. 
+ */
 static int do_redirect(Http_T *H) {
 
   char host[STRLEN];
   char auth[STRLEN]= {0};
   Port_T P= socket_get_Port(H->s);
 
+  
+  /*
+   * FIXME: there is a 
+   * BIG shortcoming here since the code only follow redirects on the 
+   * same server. What we need to do is to open a new socket against 
+   * the server in the location header. This may provide problems since 
+   * the socket is owned by the caller. Anyway, do investigate this further.
+   */
+  
   if(!*H->location) {
     DEBUG("HTTP error: Missing Location header in response\n");
     return FALSE;
