@@ -47,6 +47,7 @@ static Protocol_T myldap3= NULL;
 static Protocol_T mymysql= NULL;
 static Protocol_T mynntp= NULL;
 static Protocol_T myntp3= NULL;
+static Protocol_T mypostfix_policy= NULL;
 static Protocol_T mypop= NULL;
 static Protocol_T mysmtp= NULL;
 static Protocol_T myssh= NULL;
@@ -60,7 +61,7 @@ static Protocol_T myrsync= NULL;
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: protocol.c,v 1.23 2005/04/03 11:56:51 martinp Exp $
+ *  @version \$Id: protocol.c,v 1.24 2005/08/04 10:38:43 martinp Exp $
  *
  *  @file
  */
@@ -84,6 +85,7 @@ void gc_protocols() {
   FREE(mymysql);
   FREE(mynntp);
   FREE(myntp3);
+  FREE(mypostfix_policy);
   FREE(mypop);
   FREE(mysmtp);
   FREE(myssh);
@@ -223,6 +225,16 @@ void *create_ntp3() {
 }
 
 
+void *create_postfix_policy() {
+  if(mypostfix_policy == NULL) {
+    NEW(mypostfix_policy);
+    mypostfix_policy->name= "POSTFIX-POLICY";
+    mypostfix_policy->check= check_postfix_policy;
+  }
+  return mypostfix_policy;
+}
+
+
 void *create_pop() {
   if(mypop == NULL) {
     NEW(mypop);
