@@ -45,6 +45,7 @@
 #define P_NTP3           16
 #define P_MYSQL          17
 #define P_DNS            18
+#define P_POSTFIXPOLICY  19
 
 void  gc_protocols();
 
@@ -62,6 +63,7 @@ void* create_ldap3();
 void* create_mysql();
 void* create_nntp();
 void* create_ntp3();
+void* create_postfix_policy();
 void* create_pop();
 void* create_smtp();
 void* create_ssh();
@@ -82,6 +84,7 @@ int check_ldap3(Socket_T);
 int check_mysql(Socket_T);
 int check_nntp(Socket_T);
 int check_ntp3(Socket_T);
+int check_postfix_policy(Socket_T);
 int check_pop(Socket_T);
 int check_smtp(Socket_T);
 int check_ssh(Socket_T);
