@@ -40,48 +40,52 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Michael Amster, <mamster@webeasy.com>
  *
- *  @version \$Id: smtp.c,v 1.17 2005/01/06 20:51:50 martinp Exp $
+ *  @version \$Id: smtp.c,v 1.22 2005/09/26 19:23:06 martinp Exp $
  *
  *  @file
  */
 int check_smtp(Socket_T s) {
 
   int status;
+  char separator;
   char buf[STRLEN];
-  char msg[STRLEN];
 
   ASSERT(s);
 
-  if(socket_read(s, buf, sizeof(buf)) <= 0) {
-    log("SMTP: error receiving data -- %s\n", STRERROR);
-    return FALSE;
-  }
+  /* Read (multiline) SMTP greeting */
+  do {
+    if(socket_readln(s, buf, sizeof(buf)) <= 0) {
+      log("SMTP: error receiving data -- %s\n", STRERROR);
+      return FALSE;
+    }
+    Util_chomp(buf);
+    sscanf(buf, "%d%c%*s", &status, &separator);
+    if(status != 220) {
+      log("SMTP error: %s\n", buf);
+      return FALSE;
+    }
+  } while(separator == '-');
 
-  Util_chomp(buf);
   
-  sscanf(buf, "%d %s", &status, msg);
-  if(status != 220) {
-    log("SMTP error: %s\n", buf);
-    return FALSE;
-  }
-
+  /* Terminate the session */
   if(socket_print(s, "QUIT\r\n") < 0) {
     log("SMTP: error sending data -- %s\n", STRERROR);
     return FALSE;
   }
 
-  if(socket_read(s, buf, sizeof(buf)) <= 0) {
-    log("SMTP: error receiving data -- %s\n", STRERROR);
-    return FALSE;
-  }
-
-  Util_chomp(buf);
-  
-  sscanf(buf, "%d %s", &status, msg);
-  if(status != 221) {
-    log("SMTP error: %s\n", buf);
-    return FALSE;
-  }
+  /* Check the reply */
+  do {
+    if(socket_readln(s, buf, sizeof(buf)) <= 0) {
+      log("SMTP: error receiving data -- %s\n", STRERROR);
+      return FALSE;
+    }
+    Util_chomp(buf);
+    sscanf(buf, "%d%c%*s", &status, &separator);
+    if(status != 221) {
+      log("SMTP error: %s\n", buf);
+      return FALSE;
+    }
+  } while(separator == '-');
 
   return TRUE;
   
