@@ -150,7 +150,7 @@ static void handle_ssl_error(char *);
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: ssl.c,v 1.40 2005/02/15 19:20:09 martinp Exp $
+ *  @version \$Id: ssl.c,v 1.42 2005/08/05 09:38:09 chopp Exp $
  *
  *  @file
  */
@@ -451,7 +451,7 @@ int close_ssl_socket(ssl_connection *ssl) {
 
 
 /**
- * Garbage collection for non reusabe parts of the ssl connection
+ * Garbage collection for non reusable parts of the ssl connection
  * @param ssl ssl connection
  * @return TRUE, or FALSE if an error has occured.
  */
@@ -1135,7 +1135,7 @@ ssl_connection *accept_ssl_socket(ssl_server_connection *ssl_server) {
 #ifdef HAVE_OPENSSL
 
   int no_crypt_socket;
-  int len= sizeof(struct sockaddr_in);
+  unsigned int len= sizeof(struct sockaddr_in);
   struct sockaddr_in in;
 
   ASSERT(ssl_server);
