@@ -119,7 +119,7 @@ static int is_url_unsafe(unsigned char *);
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *  @author Michael Amster, <mamster@webeasy.com> 
  *
- *  @version \$Id: util.c,v 1.158 2005/04/11 22:27:53 martinp Exp $
+ *  @version \$Id: util.c,v 1.164 2005/09/25 21:02:54 martinp Exp $
  *  @file
  */
 
@@ -582,6 +582,8 @@ void Util_printRunList() {
       printf(" %-18s = %s\n", "Alert mail to", is_str_defined(list->to));
       printf(" %-18s = ", "Alert on");
       printevents(list->events);
+      if(list->reminder)
+        printf(" %-18s = %u cycles", "Alert reminder", list->reminder);
     }
   }
 
@@ -603,8 +605,11 @@ void Util_printService(Service_T s) {
   Resource_T q;
   Timestamp_T t;
   Size_T sl;
+  Match_T ml;
   Dependant_T d;
   char string[STRLEN];
+  char ratio1[STRLEN];
+  char ratio2[STRLEN];
 
   ASSERT(s);
  
@@ -638,57 +643,82 @@ void Util_printService(Service_T s) {
       printf(" %-20s = %s\n", "Depends on Service", d->dependant);
 
   if(s->type == TYPE_PROCESS) {
-    printf(" %-20s = if changed then %s\n",
-      "Pid", actionnames[s->action_PID->failed->id]);
-    printf(" %-20s = if changed then %s\n",
-      "Ppid", actionnames[s->action_PPID->failed->id]);
+    printf(" %-20s = if changed %s then %s\n",
+      "Pid",
+      Util_getEventratio(s->action_PID->failed, ratio1),
+      actionnames[s->action_PID->failed->id]);
+    printf(" %-20s = if changed %s then %s\n",
+      "Ppid",
+      Util_getEventratio(s->action_PPID->failed, ratio1),
+      actionnames[s->action_PPID->failed->id]);
   }
 
   if(s->checksum && s->checksum->action) {
     Checksum_T cs= s->checksum;
     EventAction_T a= cs->action;
 
+    Util_getEventratio(a->failed, ratio1);
+    Util_getEventratio(a->passed, ratio2);
+
     if(cs->test_changes)
-      printf(" %-20s = if changed %s then %s\n",
-        "Checksum", checksumnames[cs->type], actionnames[a->failed->id]);
+      printf(" %-20s = if changed %s %s then %s\n",
+        "Checksum", checksumnames[cs->type],
+        ratio1, actionnames[a->failed->id]);
     else
-      printf(" %-20s = if failed %s(%s) then %s else if recovered %s\n",
+      printf(" %-20s = if failed %s(%s) %s then %s else if passed %s then %s\n",
         "Checksum", cs->hash, checksumnames[cs->type],
-        actionnames[a->failed->id], actionnames[a->passed->id]);
+        ratio1, actionnames[a->failed->id],
+        ratio2, actionnames[a->passed->id]);
   }
   
   if(s->perm && s->perm->action) {
     EventAction_T a= s->perm->action;
 
-    printf(" %-20s = if failed %o then %s else if recovered then %s\n",
-      "Permission", s->perm->perm, actionnames[a->failed->id],
-      actionnames[a->passed->id]);
+    Util_getEventratio(a->failed, ratio1);
+    Util_getEventratio(a->passed, ratio2);
+
+    printf(" %-20s = if failed %o %s then %s else if passed %s then %s\n",
+      "Permission", s->perm->perm,
+      ratio1, actionnames[a->failed->id],
+      ratio2, actionnames[a->passed->id]);
   }
 
   if(s->uid && s->uid->action) {
     EventAction_T a= s->uid->action;
 
-    printf(" %-20s = if failed %d then %s else if recovered then %s\n",
-      "UID", (int)s->uid->uid, actionnames[a->failed->id],
-      actionnames[a->passed->id]);
+    Util_getEventratio(a->failed, ratio1);
+    Util_getEventratio(a->passed, ratio2);
+
+    printf(" %-20s = if failed %d %s then %s else if passed %s then %s\n",
+      "UID", (int)s->uid->uid,
+      ratio1, actionnames[a->failed->id],
+      ratio2, actionnames[a->passed->id]);
   }
 
   if(s->gid && s->gid->action) {
     EventAction_T a= s->gid->action;
 
-    printf(" %-20s = if failed %d then %s else if recovered then %s\n",
-      "GID", (int)s->gid->gid, actionnames[a->failed->id],
-      actionnames[a->passed->id]);
+    Util_getEventratio(a->failed, ratio1);
+    Util_getEventratio(a->passed, ratio2);
+
+    printf(" %-20s = if failed %d %s then %s else if passed %s then %s\n",
+      "GID", (int)s->gid->gid,
+      ratio1, actionnames[a->failed->id],
+      ratio2, actionnames[a->passed->id]);
   }
 
   if(s->icmplist)
     for(i= s->icmplist; i; i= i->next) {
       EventAction_T a= i->action;
 
-      printf(" %-20s = if failed %s with timeout %d seconds then %s "
-        "else if recovered then %s\n",
-        "ICMP", icmpnames[i->type], i->timeout, actionnames[a->failed->id],
-        actionnames[a->passed->id]);
+      Util_getEventratio(a->failed, ratio1);
+      Util_getEventratio(a->passed, ratio2);
+
+      printf(" %-20s = if failed %s count %d with timeout %d seconds %s then %s "
+        "else if passed %s then %s\n",
+        "ICMP", icmpnames[i->type], i->count, i->timeout,
+        ratio1, actionnames[a->failed->id],
+        ratio2, actionnames[a->passed->id]);
     }
 
   if(s->portlist) {
@@ -696,35 +726,41 @@ void Util_printService(Service_T s) {
     for(n= s->portlist; n; n= n->next) {
       EventAction_T a= n->action;
       
+      Util_getEventratio(a->failed, ratio1);
+      Util_getEventratio(a->passed, ratio2);
+
       if(n->family == AF_INET) {
 	
-	if(n->SSL.use_ssl) {
+	   if(n->SSL.use_ssl) {
 	  
-	  printf(" %-20s = if failed %s:%d%s [protocol %s via SSL] with "
-                "timeout %d seconds then %s else if recovered then %s\n",
-	        "Port", n->hostname, n->port, n->request?n->request:"",
-		 n->protocol->name, n->timeout, actionnames[a->failed->id],
-                 actionnames[a->passed->id]);
+	     printf(" %-20s = if failed %s:%d%s [protocol %s via SSL] with "
+               "timeout %d seconds %s then %s else if passed %s then %s\n",
+	       "Port", n->hostname, n->port, n->request?n->request:"",
+	       n->protocol->name, n->timeout,
+               ratio1, actionnames[a->failed->id],
+               ratio2, actionnames[a->passed->id]);
 	  
-	  if(n->SSL.certmd5 != NULL)
-	      printf(" %-20s = %s\n", "Server cert md5 sum", n->SSL.certmd5);
+	     if(n->SSL.certmd5 != NULL)
+	       printf(" %-20s = %s\n", "Server cert md5 sum", n->SSL.certmd5);
 	  
         } else {
 
-	  printf(" %-20s = if failed %s:%d%s [protocol %s] with timeout %d "
-                 "seconds then %s else if recovered then %s\n",
-	         "Port", n->hostname, n->port, n->request?n->request:"",
-		 n->protocol->name, n->timeout, actionnames[a->failed->id],
-                 actionnames[a->passed->id]);
+	     printf(" %-20s = if failed %s:%d%s [protocol %s] with timeout %d "
+               "seconds %s then %s else if passed %s then %s\n",
+               "Port", n->hostname, n->port, n->request?n->request:"",
+               n->protocol->name, n->timeout,
+               ratio1, actionnames[a->failed->id],
+               ratio2, actionnames[a->passed->id]);
 	  
-	}
+	   }
 	
       } else if(n->family == AF_UNIX) {
 	
         printf(" %-20s = if failed %s [protocol %s] with timeout %d seconds "
-               "then %s else if recovered then %s\n",
-	       "Unix Socket", n->pathname, n->protocol->name, n->timeout,
-               actionnames[a->failed->id], actionnames[a->passed->id]);
+          "%s then %s else if passed %s then %s\n",
+          "Unix Socket", n->pathname, n->protocol->name, n->timeout,
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
 	
       }
       
@@ -735,62 +771,90 @@ void Util_printService(Service_T s) {
   for(t= s->timestamplist; t; t= t->next) {
     EventAction_T a= t->action;
 
+    Util_getEventratio(a->failed, ratio1);
+    Util_getEventratio(a->passed, ratio2);
+
     if(t->test_changes)
-      printf(" %-20s = if changed then %s\n",
-        "Timestamp", actionnames[a->failed->id]);
+      printf(" %-20s = if changed %s then %s\n",
+        "Timestamp",
+        ratio1, actionnames[a->failed->id]);
     else
-      printf(" %-20s = if %s %d second(s) then %s else if recovered then %s\n",
+      printf(" %-20s = if %s %d second(s) %s then %s else if passed %s then %s\n",
         "Timestamp", operatornames[t->operator], t->time,
-        actionnames[a->failed->id], actionnames[a->passed->id]);
+        ratio1, actionnames[a->failed->id],
+        ratio2, actionnames[a->passed->id]);
     
   }
 
   for(sl= s->sizelist; sl; sl= sl->next) {
     EventAction_T a= sl->action;
     
+    Util_getEventratio(a->failed, ratio1);
+    Util_getEventratio(a->passed, ratio2);
+
     if(sl->test_changes)
-      printf(" %-20s = if changed then %s\n",
-        "Size", actionnames[a->failed->id]);
+      printf(" %-20s = if changed %s then %s\n",
+        "Size",
+        ratio1, actionnames[a->failed->id]);
     else
-      printf(" %-20s = if %s %lu byte(s) then %s else if recovered then %s\n",
+      printf(" %-20s = if %s %lu byte(s) %s then %s else if passed %s then %s\n",
         "Size", operatornames[sl->operator], sl->size,
-        actionnames[a->failed->id], actionnames[a->passed->id]);
+        ratio1, actionnames[a->failed->id],
+        ratio2, actionnames[a->passed->id]);
     
   }
 
+  for(ml= s->matchlist; ml; ml= ml->next) {
+    EventAction_T a= ml->action;
+    
+    Util_getEventratio(a->failed, ratio1);
+
+    printf(" %-20s = if%s match \"%s\" %s then %s\n",
+           "Regex", ml->not?" not":"", ml->match_string,
+           ratio1, actionnames[a->failed->id]);
+    
+  }
+  
   for(dl= s->devicelist; dl; dl= dl->next) {
     EventAction_T a= dl->action;
     
+    Util_getEventratio(a->failed, ratio1);
+    Util_getEventratio(a->passed, ratio2);
+
     if(dl->resource == RESOURCE_ID_INODE) {
 	    
       if(dl->limit_absolute > -1) {
-        printf(" %-20s = if %s %ld then %s else if recovered then %s\n",
+        printf(" %-20s = if %s %ld %s then %s else if passed %s then %s\n",
           "Inodes usage limit",
           operatornames[dl->operator],
           dl->limit_absolute,
-          actionnames[a->failed->id], actionnames[a->passed->id]);
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
       } else {
-        printf(" %-20s = if %s %.1f%% then %s else if recovered then %s\n",
+        printf(" %-20s = if %s %.1f%% %s then %s else if passed %s then %s\n",
           "Inodes usage limit",
           operatornames[dl->operator],
           dl->limit_percent/10.,
-          actionnames[a->failed->id], actionnames[a->passed->id]);
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
       }
       
     } else if(dl->resource == RESOURCE_ID_SPACE) {
 	    
       if(dl->limit_absolute > -1) {
-        printf(" %-20s = if %s %ld blocks then %s else if recovered then %s\n",
+        printf(" %-20s = if %s %ld blocks %s then %s else if passed %s then %s\n",
           "Space usage limit",
           operatornames[dl->operator],
           dl->limit_absolute,
-          actionnames[a->failed->id], actionnames[a->passed->id]);
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
       } else {
-        printf(" %-20s = if %s %.1f%% then %s else if recovered then %s\n",
+        printf(" %-20s = if %s %.1f%% %s then %s else if passed %s then %s\n",
           "Space usage limit",
           operatornames[dl->operator],
           dl->limit_percent/10.,
-          actionnames[a->failed->id], actionnames[a->passed->id]);
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
       }
 
     }
@@ -800,86 +864,98 @@ void Util_printService(Service_T s) {
   for(q= s->resourcelist; q; q= q->next) {
     EventAction_T a= q->action;
 
+    Util_getEventratio(a->failed, ratio1);
+    Util_getEventratio(a->passed, ratio2);
+
     switch(q->resource_id) {
 
     case RESOURCE_ID_CPU_PERCENT: 
 
-      printf(" %-20s = if %s %.1f%% for %d cycle(s) then %s "
-        "else if recovered then %s\n", 
+      printf(" %-20s = if %s %.1f%% %s then %s "
+        "else if passed %s then %s\n", 
         "CPU usage limit", 
-        operatornames[q->operator], q->limit/10.0, q->max_cycle,
-        actionnames[a->failed->id], actionnames[a->passed->id]);
+        operatornames[q->operator], q->limit/10.0,
+        ratio1, actionnames[a->failed->id],
+        ratio2, actionnames[a->passed->id]);
       break;
 
     case RESOURCE_ID_MEM_PERCENT: 
 
-      printf(" %-20s = if %s %.1f%% for %d cycle(s) then %s "
-        "else if recovered then %s\n", 
+      printf(" %-20s = if %s %.1f%% %s then %s "
+        "else if passed %s then %s\n", 
         "Memory usage limit", 
-        operatornames[q->operator], q->limit/10.0, q->max_cycle,
-        actionnames[a->failed->id], actionnames[a->passed->id]);
+        operatornames[q->operator], q->limit/10.0,
+        ratio1, actionnames[a->failed->id],
+        ratio2, actionnames[a->passed->id]);
       break;
 
     case RESOURCE_ID_MEM_KBYTE: 
 
-      printf(" %-20s = if %s %ldkB for %d cycle(s) then %s "
-        "else if recovered then %s\n", 
+      printf(" %-20s = if %s %ldkB %s then %s "
+        "else if passed %s then %s\n", 
         "Memory amount limit", 
-        operatornames[q->operator], q->limit, q->max_cycle,
-        actionnames[a->failed->id], actionnames[a->passed->id]);
+        operatornames[q->operator], q->limit,
+        ratio1, actionnames[a->failed->id],
+        ratio2, actionnames[a->passed->id]);
       break;
 
     case RESOURCE_ID_LOAD1: 
 
-      printf(" %-20s = if %s %.1f for %d cycle(s) then %s "
-        "else if recovered then %s\n", 
+      printf(" %-20s = if %s %.1f %s then %s "
+        "else if passed %s then %s\n", 
         "Load avg. (1min)", 
-        operatornames[q->operator], q->limit/10.0, q->max_cycle, 
-        actionnames[a->failed->id], actionnames[a->passed->id]);
+        operatornames[q->operator], q->limit/10.0,
+        ratio1, actionnames[a->failed->id],
+        ratio2, actionnames[a->passed->id]);
       break;
 
     case RESOURCE_ID_LOAD5: 
 
-      printf(" %-20s = if %s %.1f for %d cycle(s) then %s "
-        "else if recovered then %s\n", 
+      printf(" %-20s = if %s %.1f %s then %s "
+        "else if passed %s then %s\n", 
         "Load avg. (5min)", 
-        operatornames[q->operator], q->limit/10.0, q->max_cycle, 
-        actionnames[a->failed->id], actionnames[a->passed->id]);
+        operatornames[q->operator], q->limit/10.0,
+        ratio1, actionnames[a->failed->id],
+        ratio2, actionnames[a->passed->id]);
       break;
 
     case RESOURCE_ID_LOAD15: 
 
-      printf(" %-20s = if %s %.1f for %d cycle(s) then %s "
-        "else if recovered then %s\n", 
+      printf(" %-20s = if %s %.1f %s then %s "
+        "else if passed %s then %s\n", 
         "Load avg. (15min)", 
-        operatornames[q->operator], q->limit/10.0, q->max_cycle,
-        actionnames[a->failed->id], actionnames[a->passed->id]);
+        operatornames[q->operator], q->limit/10.0,
+        ratio1, actionnames[a->failed->id],
+        ratio2, actionnames[a->passed->id]);
       break;
 
     case RESOURCE_ID_CHILDREN:
 
-      printf(" %-20s = If %s %ld for %d cycle(s) then %s "
-        "else if recovered then %s\n",
-        "Children", operatornames[q->operator], q->limit, q->max_cycle,
-        actionnames[a->failed->id], actionnames[a->passed->id]);
+      printf(" %-20s = If %s %ld %s then %s "
+        "else if passed %s then %s\n",
+        "Children", operatornames[q->operator], q->limit,
+        ratio1, actionnames[a->failed->id],
+        ratio2, actionnames[a->passed->id]);
       break;
 
     case RESOURCE_ID_TOTAL_MEM_KBYTE:
 
-      printf(" %-20s = If %s %ld for %d cycle(s) then %s "
-        "else if recovered then %s\n",
+      printf(" %-20s = If %s %ld %s then %s "
+        "else if passed %s then %s\n",
         "Memory amount limit (incl. children)",
-        operatornames[q->operator], q->limit, q->max_cycle,
-        actionnames[a->failed->id], actionnames[a->passed->id]);
+        operatornames[q->operator], q->limit,
+        ratio1, actionnames[a->failed->id],
+        ratio2, actionnames[a->passed->id]);
       break;
 
     case RESOURCE_ID_TOTAL_MEM_PERCENT:
 
-      printf(" %-20s = If %s %.1f%% for %d cycle(s) then %s "
-        "else if recovered then %s\n",
+      printf(" %-20s = If %s %.1f%% %s then %s "
+        "else if passed %s then %s\n",
         "Memory usage limit (incl. children)",
-        operatornames[q->operator], q->limit/10.0, q->max_cycle,
-        actionnames[a->failed->id], actionnames[a->passed->id]);
+        operatornames[q->operator], q->limit/10.0,
+        ratio1, actionnames[a->failed->id],
+        ratio2, actionnames[a->passed->id]);
       break;
 
     }    
@@ -891,7 +967,7 @@ void Util_printService(Service_T s) {
   if(s->def_timeout && s->action_TIMEOUT) {
     EventAction_T a= s->action_TIMEOUT;
     printf(" %-20s = If %d restart within %d cycles then %s "
-      "else if recovered then %s\n",
+      "else if passed then %s\n",
       "Timeout", s->to_start, s->to_cycle, actionnames[a->failed->id],
       actionnames[a->passed->id]);
   }
@@ -900,6 +976,8 @@ void Util_printService(Service_T s) {
     printf(" %-20s = %s\n", "Alert mail to", is_str_defined(r->to));
     printf(" %-20s = ", "Alert on");
     printevents(r->events);
+    if(r->reminder)
+      printf(" %-20s = %u cycles", "Alert reminder", r->reminder);
   }
 
   printf("\n");
@@ -1528,11 +1606,14 @@ process_partial_block:
  * Reset the service information structure
  */
 void Util_resetInfo(Service_T s) {
+  FREE(s->inf->cs_sum);
   memset(s->inf, 0, sizeof *(s->inf));
-  s->inf->_pid=  -1;
-  s->inf->_ppid= -1;
-  s->inf->pid=   -1;
-  s->inf->ppid=  -1;
+  s->inf->_pid=        -1;
+  s->inf->_ppid=       -1;
+  s->inf->pid=         -1;
+  s->inf->ppid=        -1;
+  s->inf->st_ino_prev=  0;
+  s->inf->readpos=      0;
 }
 
 
@@ -1667,6 +1748,21 @@ int Util_getAction(const char *action) {
   return ACTION_IGNORE;
 }
 
+
+/**
+ * Print event ratio needed to trigger the action to the buffer
+ * @param action A action string
+ * @return the action id
+ */
+char *Util_getEventratio(Action_T action, char *buf) {
+  snprintf(buf, STRLEN,
+    "%d times within %d cycle(s)",
+    action->count, action->cycles);
+
+  return buf;
+}
+
+
 /* ----------------------------------------------------------------- Private */
 
 
@@ -1737,6 +1833,8 @@ static void printevents(unsigned int events) {
       printf("Icmp ");
     if(IS_EVENT_SET(events, EVENT_INVALID))
       printf("Invalid ");
+    if(IS_EVENT_SET(events, EVENT_MATCH))
+      printf("Match ");
     if(IS_EVENT_SET(events, EVENT_NONEXIST))
       printf("Nonexist ");
     if(IS_EVENT_SET(events, EVENT_PERMISSION))
@@ -1755,3 +1853,4 @@ static void printevents(unsigned int events) {
   printf("\n");
 }
 
+
