@@ -30,7 +30,7 @@
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *  @author Michael Amster, <mamster@webeasy.com> 
  *
- *  @version \$Id: util.h,v 1.10 2005/04/11 22:27:53 martinp Exp $
+ *  @version \$Id: util.h,v 1.11 2005/09/05 09:51:38 martinp Exp $
  *  @file
  */
 
@@ -387,4 +387,12 @@ void Util_monitorUnset(Service_T s);
 int Util_getAction(const char *action);
 
 
+/**
+ * Print event ratio needed to trigger the action to the buffer
+ * @param action A action string
+ * @return the action id
+ */
+char *Util_getEventratio(Action_T action, char *buf);
+
+
 #endif
