@@ -90,7 +90,7 @@
  *  @author Martin Pala <martinp@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: validate.c,v 1.138 2005/04/13 07:37:46 martinp Exp $
+ *  @version \$Id: validate.c,v 1.145 2005/09/30 10:25:14 martinp Exp $
  *  @file
  */
 
@@ -102,6 +102,9 @@ static void check_uid(Service_T);
 static void check_gid(Service_T);
 static void check_size(Service_T);
 static void check_perm(Service_T);
+ void check_match(Service_T);
+ int  check_match_ignore(Service_T, char *);
+ void check_match_if(Service_T, char *);
 static int  check_skip(Service_T);
 static int  check_timeout(Service_T);
 static void check_checksum(Service_T);
@@ -118,6 +121,7 @@ int            ptreesize=0;
 ProcessTree_T *oldptree=NULL;  
 int            oldptreesize=0; 
 
+#define MATCH_LINE_LENGTH 512
 
 /* ---------------------------------------------------------------- Public */
 
@@ -134,8 +138,7 @@ void validate() {
 
   if(Run.doprocess) {
     initprocesstree(&ptree, &ptreesize, &oldptree, &oldptreesize);
-    if(! update_system_load(ptree, ptreesize))
-      log("Update of system load has failed!\n");
+    update_system_load(ptree, ptreesize);
   }
   time(&systeminfo.collected);
 
@@ -156,9 +159,6 @@ void validate() {
 
   reset_depend();
 
-  /* Post current status to the collector */
-  handle_collector(NULL);
-
 }
 
 
@@ -178,11 +178,11 @@ int check_process(Service_T s) {
   if(!(pid= Util_isProcessRunning(s))) {
     /* Reset the service info object to prevent false data in the first run */
     Util_resetInfo(s);
-    Event_post(s, EVENT_NONEXIST, TRUE, s->action_NONEXIST,
+    Event_post(s, EVENT_NONEXIST, STATE_FAILED, s->action_NONEXIST,
       "'%s' process is not running", s->name);
     return FALSE;
   } else {
-    Event_post(s, EVENT_NONEXIST, FALSE, s->action_NONEXIST,
+    Event_post(s, EVENT_NONEXIST, STATE_PASSED, s->action_NONEXIST,
       "'%s' process is running with pid %d", s->name, (int)pid);
   }
 
@@ -224,19 +224,19 @@ int check_device(Service_T s) {
   ASSERT(s);
 
   if(stat(s->path, &stat_buf) != 0) {
-    Event_post(s, EVENT_NONEXIST, TRUE, s->action_NONEXIST,
+    Event_post(s, EVENT_NONEXIST, STATE_FAILED, s->action_NONEXIST,
       "'%s' device doesn't exist", s->name);
     return FALSE;
   } else {
     s->inf->st_mode= stat_buf.st_mode;
     s->inf->st_uid= stat_buf.st_uid;
     s->inf->st_gid= stat_buf.st_gid;
-    Event_post(s, EVENT_NONEXIST, FALSE, s->action_NONEXIST,
+    Event_post(s, EVENT_NONEXIST, STATE_PASSED, s->action_NONEXIST,
       "'%s' device exist", s->name);
   }
 
   if(!device_usage(s->inf, s->path)) {
-    Event_post(s, EVENT_DATA, TRUE, s->action_DATA,
+    Event_post(s, EVENT_DATA, STATE_FAILED, s->action_DATA,
       "'%s' unable to read device %s state", s->name, s->path);
     return FALSE;
   } else {
@@ -248,7 +248,7 @@ int check_device(Service_T s) {
             (float)s->inf->f_blocks);
     s->inf->inode_total= s->inf->f_files - s->inf->f_filesfree;
     s->inf->space_total= s->inf->f_blocks - s->inf->f_blocksfreetotal;
-    Event_post(s, EVENT_DATA, FALSE, s->action_DATA,
+    Event_post(s, EVENT_DATA, STATE_PASSED, s->action_DATA,
       "'%s' succeeded getting device statistic for %s", s->name, s->path);
   }
 
@@ -281,27 +281,34 @@ int check_file(Service_T s) {
   ASSERT(s);
 
   if(stat(s->path, &stat_buf) != 0) {
-    Event_post(s, EVENT_NONEXIST, TRUE, s->action_NONEXIST,
+    Event_post(s, EVENT_NONEXIST, STATE_FAILED, s->action_NONEXIST,
       "'%s' file doesn't exist", s->name);
     return FALSE;
   } else {
     s->inf->st_mode= stat_buf.st_mode;
+    if (s->inf->st_ino==0) {
+      s->inf->st_ino_prev= stat_buf.st_ino;
+      s->inf->readpos= stat_buf.st_size;
+    } else {
+      s->inf->st_ino_prev= s->inf->st_ino;
+    }
+    s->inf->st_ino= stat_buf.st_ino;
     s->inf->st_uid= stat_buf.st_uid;
     s->inf->st_gid= stat_buf.st_gid;
     s->inf->st_size= stat_buf.st_size;
     s->inf->timestamp= MAX(stat_buf.st_mtime, stat_buf.st_ctime);
     DEBUG("'%s' file existence check passed\n", s->name);
-    Event_post(s, EVENT_NONEXIST, FALSE, s->action_NONEXIST,
+    Event_post(s, EVENT_NONEXIST, STATE_PASSED, s->action_NONEXIST,
       "'%s' file exist", s->name);
   }
 
   if(!S_ISREG(s->inf->st_mode)) {
-    Event_post(s, EVENT_INVALID, TRUE, s->action_INVALID,
+    Event_post(s, EVENT_INVALID, STATE_FAILED, s->action_INVALID,
       "'%s' is not regular file", s->name);
     return FALSE;
   } else {
     DEBUG("'%s' is regular file\n", s->name);
-    Event_post(s, EVENT_INVALID, FALSE, s->action_INVALID,
+    Event_post(s, EVENT_INVALID, STATE_PASSED, s->action_INVALID,
       "'%s' is regular file", s->name);
   }
 
@@ -323,6 +330,9 @@ int check_file(Service_T s) {
   if(s->timestamplist)
     check_timestamp(s);
 
+  if(s->matchlist)
+    check_match(s);
+
   return TRUE;
 
 }
@@ -339,7 +349,7 @@ int check_directory(Service_T s) {
   ASSERT(s);
 
   if(stat(s->path, &stat_buf) != 0) {
-    Event_post(s, EVENT_NONEXIST, TRUE, s->action_NONEXIST,
+    Event_post(s, EVENT_NONEXIST, STATE_FAILED, s->action_NONEXIST,
       "'%s' directory doesn't exist", s->name);
     return FALSE;
   } else {
@@ -348,17 +358,17 @@ int check_directory(Service_T s) {
     s->inf->st_gid= stat_buf.st_gid;
     s->inf->timestamp= MAX(stat_buf.st_mtime, stat_buf.st_ctime);
     DEBUG("'%s' directory existence check passed\n", s->name);
-    Event_post(s, EVENT_NONEXIST, FALSE, s->action_NONEXIST,
+    Event_post(s, EVENT_NONEXIST, STATE_PASSED, s->action_NONEXIST,
       "'%s' directory exist", s->name);
   }
 
   if(!S_ISDIR(s->inf->st_mode)) {
-    Event_post(s, EVENT_INVALID, TRUE, s->action_INVALID,
+    Event_post(s, EVENT_INVALID, STATE_FAILED, s->action_INVALID,
       "'%s' is not directory", s->name);
     return FALSE;
   } else {
     DEBUG("'%s' is directory\n", s->name);
-    Event_post(s, EVENT_INVALID, FALSE, s->action_INVALID,
+    Event_post(s, EVENT_INVALID, STATE_PASSED, s->action_INVALID,
       "'%s' is directory", s->name);
   }
 
@@ -386,8 +396,9 @@ int check_directory(Service_T s) {
  */
 int check_remote_host(Service_T s) {
 
-  Port_T p= NULL;
-  Icmp_T icmp;
+  Port_T p = NULL;
+  Icmp_T icmp = NULL;
+  Icmp_T last_ping = NULL;
 
   ASSERT(s);
 
@@ -398,20 +409,21 @@ int check_remote_host(Service_T s) {
       switch(icmp->type) {
       case ICMP_ECHO:
 
-        icmp->response= icmp_echo(s->path, icmp->timeout);
+        icmp->response= icmp_echo(s->path, icmp->timeout, icmp->count);
 
         if(icmp->response < 0) {
           icmp->is_available= FALSE;
           DEBUG("'%s' icmp ping failed\n", s->name);
-          Event_post(s, EVENT_ICMP, TRUE, icmp->action,
+          Event_post(s, EVENT_ICMP, STATE_FAILED, icmp->action,
             "'%s' failed ICMP test [%s]", s->name, icmpnames[icmp->type]);
         } else {
           icmp->is_available= TRUE;
           DEBUG("'%s' icmp ping succeeded [response time %.3fs]\n",
             s->name, icmp->response);
-          Event_post(s, EVENT_ICMP, FALSE, icmp->action,
+          Event_post(s, EVENT_ICMP, STATE_PASSED, icmp->action,
             "'%s' passed ICMP test [%s]", s->name, icmpnames[icmp->type]);
         }
+        last_ping = icmp;
         break;
 
       default:
@@ -422,6 +434,14 @@ int check_remote_host(Service_T s) {
     }
   }
 
+  /* If we could not ping the host we assume it's down and do not
+   * continue to check any port connections  */
+  if(last_ping && !last_ping->is_available) {
+    DEBUG("'%s' icmp ping failed, skipping any port connection tests\n",
+      s->name);
+    return FALSE;
+  }
+
   /* Test each host:port and protocol in the service's portlist */
   if(s->portlist)
     for(p= s->portlist; p; p= p->next)
@@ -499,10 +519,10 @@ static void check_connection(Service_T s, Port_T p) {
 
   if(!rv) {
     p->is_available= FALSE;
-    Event_post(s, EVENT_CONNECTION, TRUE, p->action, report);
+    Event_post(s, EVENT_CONNECTION, STATE_FAILED, p->action, report);
   } else {
     p->is_available= TRUE;
-    Event_post(s, EVENT_CONNECTION, FALSE, p->action,
+    Event_post(s, EVENT_CONNECTION, STATE_PASSED, p->action,
       "'%s' connection passed", s->name);
   }
       
@@ -520,12 +540,12 @@ static void check_process_state(Service_T s) {
     /* We do not check the process anymore if it's a zombie
        since such a process is (usually) unmanageable */
     Util_monitorUnset(s);
-    Event_post(s, EVENT_DATA, TRUE, s->action_DATA,
+    Event_post(s, EVENT_DATA, STATE_FAILED, s->action_DATA,
       "'%s' process with pid %d is a zombie", s->name, s->inf->pid);
   } else {
     DEBUG("'%s' zombie check passed [status_flag=%04x]\n",
       s->name,  s->inf->status_flag);
-    Event_post(s, EVENT_DATA, FALSE, s->action_DATA,
+    Event_post(s, EVENT_DATA, STATE_PASSED, s->action_DATA,
       "'%s' check process state passed", s->name);
   }
 
@@ -544,11 +564,11 @@ static void check_process_pid(Service_T s) {
     return;
 
   if(s->inf->_pid != s->inf->pid) {
-    Event_post(s, EVENT_CHANGED, TRUE, s->action_PID,
+    Event_post(s, EVENT_CHANGED, STATE_FAILED, s->action_PID,
       "'%s' process PID changed to %d", s->name, s->inf->pid);
   } else {
     DEBUG("'%s' PID has not changed since last cycle\n", s->name);
-    Event_post(s, EVENT_CHANGED, FALSE, s->action_PID,
+    Event_post(s, EVENT_CHANGED, STATE_PASSED, s->action_PID,
       "'%s' PID has not changed", s->name);
   }
 }
@@ -566,11 +586,11 @@ static void check_process_ppid(Service_T s) {
     return;
 
   if(s->inf->_ppid != s->inf->ppid) {
-    Event_post(s, EVENT_CHANGED, TRUE, s->action_PPID,
+    Event_post(s, EVENT_CHANGED, STATE_FAILED, s->action_PPID,
       "'%s' process PPID changed to %d", s->name, s->inf->ppid);
   } else {
     DEBUG("'%s' PPID has not changed since last cycle\n", s->name);
-    Event_post(s, EVENT_CHANGED, FALSE, s->action_PPID,
+    Event_post(s, EVENT_CHANGED, STATE_PASSED, s->action_PPID,
       "'%s' PPID has not changed", s->name);
   }
 }
@@ -716,17 +736,10 @@ static void check_process_resources(Service_T s, Resource_T pr) {
     return;
   }
 
-  if(okay && pr->cycle > 0) {
-    pr->cycle--;
-  } else if(! okay) {
-    pr->cycle++;
-  }
-
-  if(pr->cycle >= pr->max_cycle) {
-    pr->cycle=0;
-    Event_post(s, EVENT_RESOURCE, TRUE, pr->action, report);
+  if(! okay) {
+    Event_post(s, EVENT_RESOURCE, STATE_FAILED, pr->action, report);
   } else {
-    Event_post(s, EVENT_RESOURCE, FALSE, pr->action,
+    Event_post(s, EVENT_RESOURCE, STATE_PASSED, pr->action,
       "'%s' resource passed", s->name);
   }
   
@@ -750,7 +763,7 @@ static void check_checksum(Service_T s) {
 
   if(s->inf->cs_sum) {
 
-    Event_post(s, EVENT_DATA, FALSE, s->action_DATA,
+    Event_post(s, EVENT_DATA, STATE_PASSED, s->action_DATA,
       "'%s' checksum computed for %s", s->name, s->path);
 
     switch(cs->type) {
@@ -770,27 +783,27 @@ static void check_checksum(Service_T s) {
 
       /* if we are testing for changes only, the value is variable */
       if(cs->test_changes) {
-        Event_post(s, EVENT_CHANGED, TRUE, cs->action,
+        Event_post(s, EVENT_CHANGED, STATE_FAILED, cs->action,
           "'%s' checksum was changed for %s", s->name, s->path);
         /* reset expected value for next cycle */
         FREE(cs->hash);
         cs->hash= xstrdup(s->inf->cs_sum);
       } else {
         /* we are testing constant value for failed or passed state */
-        Event_post(s, EVENT_CHECKSUM, TRUE, cs->action,
+        Event_post(s, EVENT_CHECKSUM, STATE_FAILED, cs->action,
           "'%s' checksum test failed for %s", s->name, s->path);
       }
 
     } else if(cs->test_changes) {
 
       DEBUG("'%s' checksum has not changed\n", s->name);
-      Event_post(s, EVENT_CHANGED, FALSE, cs->action,
+      Event_post(s, EVENT_CHANGED, STATE_PASSED, cs->action,
         "'%s' checksum has not changed", s->name);
 
     } else {
 
       DEBUG("'%s' has valid checksums\n", s->name);
-      Event_post(s, EVENT_CHECKSUM, FALSE, cs->action,
+      Event_post(s, EVENT_CHECKSUM, STATE_PASSED, cs->action,
         "'%s' checksum passed", s->name);
 
     }
@@ -799,7 +812,7 @@ static void check_checksum(Service_T s) {
 
   }
 
-  Event_post(s, EVENT_DATA, TRUE, s->action_DATA,
+  Event_post(s, EVENT_DATA, STATE_FAILED, s->action_DATA,
     "'%s' cannot compute checksum for %s", s->name, s->path);
 
 }
@@ -813,13 +826,13 @@ static void check_perm(Service_T s) {
   ASSERT(s && s->perm);
 
   if((s->inf->st_mode & 07777) != s->perm->perm) {
-    Event_post(s, EVENT_PERMISSION, TRUE, s->perm->action,
+    Event_post(s, EVENT_PERMISSION, STATE_FAILED, s->perm->action,
       "'%s' permission test failed for %s -- current permission is %o",
       s->name, s->path, s->inf->st_mode&07777);
   } else {
     DEBUG("'%s' permission check passed [current permission=%o]\n",
       s->name, s->inf->st_mode&07777);
-    Event_post(s, EVENT_PERMISSION, FALSE, s->perm->action,
+    Event_post(s, EVENT_PERMISSION, STATE_PASSED, s->perm->action,
       "'%s' permission passed", s->name);
   }
 }
@@ -833,13 +846,13 @@ static void check_uid(Service_T s) {
   ASSERT(s && s->uid);
 
   if(s->inf->st_uid != s->uid->uid) {
-    Event_post(s, EVENT_UID, TRUE, s->uid->action,
+    Event_post(s, EVENT_UID, STATE_FAILED, s->uid->action,
       "'%s' uid test failed for %s -- current uid is %d",
       s->name, s->path, (int)s->inf->st_uid);
   } else {
     DEBUG("'%s' uid check passed [current uid=%d]\n", s->name,
           (int)s->inf->st_uid);
-    Event_post(s, EVENT_UID, FALSE, s->uid->action, "'%s' uid passed", s->name);
+    Event_post(s, EVENT_UID, STATE_PASSED, s->uid->action, "'%s' uid passed", s->name);
   }
 
 }
@@ -853,13 +866,13 @@ static void check_gid(Service_T s) {
   ASSERT(s && s->gid);
 
   if(s->inf->st_gid != s->gid->gid ) {
-    Event_post(s, EVENT_GID, TRUE, s->gid->action,
+    Event_post(s, EVENT_GID, STATE_FAILED, s->gid->action,
       "'%s' gid test failed for %s -- current gid is %d",
       s->name, s->path, (int)s->inf->st_gid);
   } else {
     DEBUG("'%s' gid check passed [current gid=%d]\n", s->name,
           (int)s->inf->st_gid);
-    Event_post(s, EVENT_GID, FALSE, s->gid->action, "'%s' gid passed", s->name);
+    Event_post(s, EVENT_GID, STATE_PASSED, s->gid->action, "'%s' gid passed", s->name);
   }
 
 }
@@ -877,11 +890,11 @@ static void check_timestamp(Service_T s) {
 
 
   if((int)time(&now) == -1) {
-    Event_post(s, EVENT_DATA, TRUE, s->action_DATA,
+    Event_post(s, EVENT_DATA, STATE_FAILED, s->action_DATA,
       "'%s' can't obtain actual system time", s->name);
     return;
   } else {
-    Event_post(s, EVENT_DATA, FALSE, s->action_DATA,
+    Event_post(s, EVENT_DATA, STATE_PASSED, s->action_DATA,
       "'%s' actual system time obtained", s->name);
   }
 
@@ -889,24 +902,24 @@ static void check_timestamp(Service_T s) {
     if(t->test_changes) {
     /* if we are testing for changes only, the value is variable */
       if(t->timestamp != s->inf->timestamp) {
-        Event_post(s, EVENT_CHANGED, TRUE, t->action,
+        Event_post(s, EVENT_CHANGED, STATE_FAILED, t->action,
           "'%s' timestamp was changed for %s", s->name, s->path);
         /* reset expected value for next cycle */
         t->timestamp= s->inf->timestamp;
       } else {
         DEBUG("'%s' timestamp was not changed for %s\n", s->name, s->path);
-        Event_post(s, EVENT_CHANGED, FALSE, t->action,
+        Event_post(s, EVENT_CHANGED, STATE_PASSED, t->action,
           "'%s' timestamp was not changed for %s", s->name, s->path);
       }
       break;
     } else {
     /* we are testing constant value for failed or passed state */
       if(Util_evalQExpression(t->operator, (int)(now - s->inf->timestamp), t->time)) {
-        Event_post(s, EVENT_TIMESTAMP, TRUE, t->action,
+        Event_post(s, EVENT_TIMESTAMP, STATE_FAILED, t->action,
           "'%s' timestamp test failed for %s", s->name, s->path);
       } else {
         DEBUG("'%s' timestamp test passed for %s\n", s->name, s->path); 
-        Event_post(s, EVENT_TIMESTAMP, FALSE, t->action,
+        Event_post(s, EVENT_TIMESTAMP, STATE_PASSED, t->action,
           "'%s' timestamp passed", s->name);
       }
     }
@@ -928,14 +941,14 @@ static void check_size(Service_T s) {
     /* if we are testing for changes only, the value is variable */
     if(sl->test_changes) {
       if(sl->size != s->inf->st_size) {
-        Event_post(s, EVENT_CHANGED, TRUE, sl->action,
+        Event_post(s, EVENT_CHANGED, STATE_FAILED, sl->action,
           "'%s' size was changed for %s", s->name, s->path);
         /* reset expected value for next cycle */
         sl->size= s->inf->st_size;
       } else {
         DEBUG("'%s' size has not changed [current size=%lu B]\n", s->name,
               s->inf->st_size);
-        Event_post(s, EVENT_CHANGED, FALSE, sl->action,
+        Event_post(s, EVENT_CHANGED, STATE_PASSED, sl->action,
           "'%s' size was not changed", s->name, s->path);
       }
       break;
@@ -943,17 +956,224 @@ static void check_size(Service_T s) {
 
     /* we are testing constant value for failed or passed state */
     if(Util_evalQExpression(sl->operator, s->inf->st_size, sl->size)) {
-      Event_post(s, EVENT_SIZE, TRUE, sl->action,
+      Event_post(s, EVENT_SIZE, STATE_FAILED, sl->action,
         "'%s' size test failed for %s -- current size is %lu B",
         s->name, s->path, s->inf->st_size);
     } else {
       DEBUG("'%s' file size check passed [current size=%lu B]\n", s->name,
             s->inf->st_size);
-      Event_post(s, EVENT_SIZE, FALSE, sl->action, "'%s' size passed", s->name);
+      Event_post(s, EVENT_SIZE, STATE_PASSED, sl->action, "'%s' size passed", s->name);
     }
   }
 }
 
+/**
+ * Match content
+ */
+ void check_match(Service_T s) {
+  char    line[MATCH_LINE_LENGTH];
+  FILE    *file;
+  int     inode_checked=FALSE;
+  int     advance=0;
+  int     ignore;
+    
+  ASSERT(s && s->matchlist);
+
+  /* did inode change -> read position = 0 */
+  if((inode_checked==FALSE) && (s->inf->st_ino != s->inf->st_ino_prev)) {
+    s->inf->readpos= 0;
+  }
+  inode_checked= TRUE;
+  
+  /* did file decrease (readpos > file_size) -> read position = 0 */
+  if(s->inf->readpos > s->inf->st_size) {
+    s->inf->readpos= 0;
+  }
+
+  /* Do we need to match? (readpos < file_size) */
+  if(!(s->inf->readpos < s->inf->st_size)){
+    return;
+  }
+
+  /* Open the file */
+  if(NULL==(file=fopen(s->path, "r"))) {
+    /* We can't open the file */
+    DEBUG("FILE: cannot open file %s: %s!\n", s->path, strerror(errno));
+    return;
+  }
+
+  while (TRUE) {
+    ignore=FALSE;
+    
+    /* Seek to the read position */
+    if (fseek(file, s->inf->readpos, SEEK_SET)!=0) {
+      /* We can not seek to the read position */
+      DEBUG("FILE: cannot seek file %s: %s!\n", s->path, strerror(errno));
+      goto final;
+    }
+
+    if(NULL==fgets(line, MATCH_LINE_LENGTH, file)) {
+      /* We can not read the content! */
+      if (!feof(file)) {
+        DEBUG("FILE: cannot read file %s: %s!\n", s->path, strerror(errno));
+      }
+      goto final;
+    }
+    
+    /* Close the file */
+    
+    /* Empty line? Should not happen... but who knows */
+    if (strlen(line) == 0) {
+      /* ==> ERROR */
+      goto final;
+    }
+
+    /* Complete line oder just beginning? (igore full buffers) */
+    if ((strlen(line)<(MATCH_LINE_LENGTH)-1) &&
+        (line[strlen(line)-1] != '\n')) {
+      /* we gonna read it next time */
+      goto final;
+    }
+
+    advance=strlen(line);
+    
+    /*
+      Does this line end with '\n'? Otherwise ignore and check it
+      as soon as it is complete
+    */
+    if (strlen(line)==(MATCH_LINE_LENGTH)-1) {
+      int  rv=0;
+
+      while (((unsigned char) rv != '\n') && (rv!=EOF)) {
+        rv=fgetc(file);
+        advance++;
+      }
+
+      if (rv==EOF) {
+        break;
+      }
+    }
+
+    /* Set read position to the end of last read */
+    s->inf->readpos+=advance;
+
+    /* Remove appending newline */
+    if (line[strlen(line)-1] == '\n') {
+      line[strlen(line)-1] = 0;
+    }
+
+    check_match_if(s, line);
+  }
+
+  final:
+  
+  fclose(file);
+}
+
+/**
+ * Match line for "ignore" statements
+ */
+ int check_match_ignore(Service_T s, char *line) {
+
+  int     rv=FALSE;
+  Match_T ml;
+  Match_T prev=NULL;
+  int     match_return;
+  
+  /* Check ignores */
+
+  for(ml= s->matchlist; ml; prev=ml, ml= ml->next) {
+    if (ml->ignore) {
+#ifdef HAVE_REGEX_H
+      match_return=regexec(ml->regex_comp,
+                           line,
+                           0,
+                           NULL,
+                           0);
+#else
+      if (strstr(line, ml->match_string) == NULL) {
+        match_return= -1;
+      } else {
+        match_return= 0;
+      }
+#endif
+      if((match_return==0)  ^ (ml->not)) {
+        /* We match! -> line is ignored! */
+        DEBUG("FILE: Regular expression %s\"%s\" "
+              "ignore match on content line\n",
+              ml->not?"not ":"",
+              ml->match_string);
+        rv=TRUE;
+        break;
+      }
+    }
+  }
+
+  /* Optimize match list => put recent match in front */
+
+  if (prev!=NULL && rv==TRUE) {
+    prev->next=ml->next;
+    ml->next=s->matchlist;
+    s->matchlist=ml;
+  }
+  
+  return rv;
+}
+
+/**
+ * Match line for "if" statements
+ */
+ void check_match_if(Service_T s, char *line) {
+
+  Match_T ml;
+  int     match_return;
+  int     ignore_tested= FALSE;
+
+  /* Check non ignores */
+  
+  for(ml= s->matchlist; ml; ml= ml->next) {
+   
+    if (!(ml->ignore)) {
+
+#ifdef HAVE_REGEX_H
+      match_return=regexec(ml->regex_comp,
+                           line,
+                           0,
+                           NULL,
+                           0);
+#else
+      if (strstr(line, ml->match_string) == NULL) {
+        match_return= -1;
+      } else {
+        match_return= 0;
+      }
+#endif
+
+      if((match_return==0) ^ (ml->not)) {
+        /* Check if we have to test for ignores! */
+        if (!ignore_tested && check_match_ignore(s, line)) {
+          return;
+        }
+        
+        Event_post(s, EVENT_MATCH, STATE_FAILED, ml->action,
+                   "'%s' content match "
+                   "[%s]",
+                   s->name, line);
+        DEBUG("FILE: Regular expression %s\"%s\" "
+              "DOES match on content line\n",
+              ml->not?"not ":"",
+              ml->match_string);
+      } else {
+        DEBUG("FILE: Regular expression %s\"%s\" "
+              "does not match on content line\n",
+              ml->not?"not ":"",
+              ml->match_string);
+      }
+    }
+  }
+  
+  return;
+}
 
 /**
  * Device test
@@ -977,7 +1197,7 @@ static void check_device_resources(Service_T s, Device_T td) {
 
       if(td->limit_percent >= 0) {
 	if(Util_evalQExpression( td->operator, s->inf->inode_percent, td->limit_percent)) {
-          Event_post(s, EVENT_RESOURCE, TRUE, td->action,
+          Event_post(s, EVENT_RESOURCE, STATE_FAILED, td->action,
             "'%s' inode usage %.1f%% matches resource limit [inode usage%s%.1f%%]",
             s->name,
 	    s->inf->inode_percent/10.,
@@ -987,7 +1207,7 @@ static void check_device_resources(Service_T s, Device_T td) {
 	}
       } else {
 	if(Util_evalQExpression(td->operator, s->inf->inode_total, td->limit_absolute)) {
-          Event_post(s, EVENT_RESOURCE, TRUE, td->action,
+          Event_post(s, EVENT_RESOURCE, STATE_FAILED, td->action,
             "'%s' inode usage %ld matches resource limit [inode usage%s%ld]",
             s->name,
             s->inf->inode_total,
@@ -998,14 +1218,14 @@ static void check_device_resources(Service_T s, Device_T td) {
       }
       DEBUG("'%s' inode usage check passed [current inode usage=%.1f%%]\n",
 	    s->name, s->inf->inode_percent/10.);
-      Event_post(s, EVENT_RESOURCE, FALSE, td->action,
+      Event_post(s, EVENT_RESOURCE, STATE_PASSED, td->action,
         "'%s' device resources passed", s->name);
       return;
 
   case RESOURCE_ID_SPACE:
       if(td->limit_percent >= 0) {
         if(Util_evalQExpression( td->operator, s->inf->space_percent, td->limit_percent)) {
-          Event_post(s, EVENT_RESOURCE, TRUE, td->action,
+          Event_post(s, EVENT_RESOURCE, STATE_FAILED, td->action,
 	    "'%s' space usage %.1f%% matches resource limit [space usage%s%.1f%%]",
             s->name,
             s->inf->space_percent/10.,
@@ -1015,7 +1235,7 @@ static void check_device_resources(Service_T s, Device_T td) {
         }
       } else {
         if(Util_evalQExpression(td->operator, s->inf->space_total, td->limit_absolute)) {
-          Event_post(s, EVENT_RESOURCE, TRUE, td->action,
+          Event_post(s, EVENT_RESOURCE, STATE_FAILED, td->action,
             "'%s' space usage %ld blocks matches resource limit "
             "[space usage%s%ld blocks]",
             s->name,
@@ -1027,7 +1247,7 @@ static void check_device_resources(Service_T s, Device_T td) {
       }
       DEBUG("'%s' space usage check passed [current space usage=%.1f%%]\n",
 	    s->name, s->inf->space_percent/10.);
-      Event_post(s, EVENT_RESOURCE, FALSE, td->action,
+      Event_post(s, EVENT_RESOURCE, STATE_PASSED, td->action,
         "'%s' device resources passed", s->name);
       return;
       
@@ -1059,7 +1279,7 @@ static int check_timeout(Service_T s) {
    * Check timeout
    */
   if(s->nstart >= s->to_start && s->ncycle <= s->to_cycle) {
-    Event_post(s, EVENT_TIMEOUT, TRUE, s->action_TIMEOUT,
+    Event_post(s, EVENT_TIMEOUT, STATE_FAILED, s->action_TIMEOUT,
               "'%s' service timed out and will not be checked anymore",
               s->name);
     return TRUE;
