@@ -50,7 +50,7 @@
  *
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: xml.c,v 1.13 2005/04/05 19:52:57 martinp Exp $
+ *  @version \$Id: xml.c,v 1.15 2005/08/05 09:38:09 chopp Exp $
  *
  *  @file
  */
@@ -61,7 +61,7 @@
 
 /** Defines an output buffer object */
 typedef struct mybuffer {
-  unsigned char *buf;                               /**< Output buffer       */
+  char          *buf;                               /**< Output buffer       */
   size_t         bufsize;                           /**< Output buffer size  */
   size_t         bufused;                           /**< Output buffer usage */
 } Buffer_T;
@@ -367,6 +367,7 @@ static void status_event(Event_T E, Buffer_T *B) {
     "\t<event>\r\n"
     "\t\t<collected>%ld</collected>\r\n"
     "\t\t<service>%s</service>\r\n"
+    "\t\t<type>%d</type>\r\n"
     "\t\t<group>%s</group>\r\n"
     "\t\t<id>%d</id>\r\n"
     "\t\t<state>%d</state>\r\n"
@@ -375,6 +376,7 @@ static void status_event(Event_T E, Buffer_T *B) {
     "\t</event>\r\n",
     Event_get_collected(E),
     s->name,
+    s->type,
     s->group?s->group:"",
     Event_get_id(E),
     Event_get_state(E),
